@@ -2,7 +2,7 @@
  *  cache-file.c
  *  mod_musicindex
  *
- *  $Id: cache-file.c 983 2012-04-07 10:38:30Z varenet $
+ *  $Id: cache-file.c 1011 2012-08-07 20:23:39Z varenet $
  *
  *  Created by Thibaut VARENE on Wed Feb 23 2005.
  *  Copyright (c) 2003-2007,2009-2010 Thibaut VARENE
@@ -18,7 +18,7 @@
  * Flat file cache management subsystem.
  *
  * @author Thibaut Varene
- * @version $Revision: 983 $
+ * @version $Revision: 1011 $
  * @date 2003-2010
  *
  * @warning 'errno' seems not to be seen as volatile by the compiler, therefore we
@@ -271,20 +271,20 @@ static void file_cache_remove_dir(request_rec *r, DIR *cachedir, const char *con
  *
  * @param r Apache request_rec struct.
  * @param pack Ignored
- * @param conf The config structure used to find out cache configuration.
- * @param names Structure containing the directory name
+ * @param filename current directory path
+ * @param uri server-side uri (unused)
  * @param soptions Flags to use for created entries. Ignored.
  *
  * @return NULL.
  */
-static void* cache_file_opendir(request_rec *r, mu_pack *const pack, const mu_config *const conf,
-				const mu_ent_names * const names, unsigned long soptions)
+static void* cache_file_opendir(request_rec *r, mu_pack *const pack,
+				const char * const filename, const char * uri, unsigned long soptions)
 {
-	const char *const path = names->filename;
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	DIR		*cachedir = NULL;
 	struct stat	cachedirstat, dirstat;
 
-	if (!path || !conf->cache_setup)
+	if (!filename || !conf->cache_setup)
 		return NULL;
 
 	/* Bear in mind we're chdir'd from now on. */
@@ -293,9 +293,9 @@ static void* cache_file_opendir(request_rec *r, mu_pack *const pack, const mu_co
 
 	/* Actually check for the directory in the cache, create it if needed.
 	 * "+ 1" offset to suppress leading '/'. */
-	if (!(cachedir = opendir(path + 1))) {			/* on essaye d'ouvrir le repertoire concerne dans le cache (on supprime le leading "/" */
+	if (!(cachedir = opendir(filename + 1))) {			/* on essaye d'ouvrir le repertoire concerne dans le cache (on supprime le leading "/" */
 		if (errno == ENOENT) {				/* il n'existe pas mais on peut le creer (ca correspond a ENOENT, a verifier) */
-			if (file_cache_make_dir(r, path + 1))	/* on envoie le chemin prive du leading '/' */
+			if (file_cache_make_dir(r, filename + 1))	/* on envoie le chemin prive du leading '/' */
 				goto error_out;
 		}
 		else
@@ -303,11 +303,11 @@ static void* cache_file_opendir(request_rec *r, mu_pack *const pack, const mu_co
 	}
 	else {	/* Checking for cache sanity. Has it expired for that folder ? If so, delete its content. */
 		fstat(dirfd(cachedir), &cachedirstat);		/* recuperons les stats du repertoire cache. XXX On considere cet appel sans echec vu les tests qu'on a fait avant. */
-		stat(path, &dirstat);				/* recuperons les stats du rep d'origine. XXX pas de test ici, a priori ya pas de raison qu'on puisse pas les recuperer */
+		stat(filename, &dirstat);				/* recuperons les stats du rep d'origine. XXX pas de test ici, a priori ya pas de raison qu'on puisse pas les recuperer */
 		if (cachedirstat.st_mtime < dirstat.st_mtime)	/* si la date de modif du rep de cache est plus vieille que celle du rep original, alors qqc a ete ajoute ou retire ou ecrit */
-			file_cache_remove_dir(r, cachedir, path);	/* alors on le vide proprement de son contenu */
+			file_cache_remove_dir(r, cachedir, filename);	/* alors on le vide proprement de son contenu */
 		closedir(cachedir);				/* On en a fini avec le repertoire, on le referme */
-		if (file_cache_make_dir(r, path + 1))	/* on recree le rep */
+		if (file_cache_make_dir(r, filename + 1))	/* on recree le rep */
 			goto error_out;		
 	}
 
@@ -337,15 +337,15 @@ struct mi_data_buffer {
  * @param r Apache request_rec struct to handle log writings.
  * @param pool Apache pool
  * @param in Not used (except for closing).
- * @param conf MusicIndex configuration paramaters struct.
- * @param names struct names to get the current filename.
+ * @param filename current filename.
  *
  * @return When possible, struct mu_ent correctly set up, file stream closed,
  * 		NULL otherwise.
  */
 static mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	const mu_config *const conf, mu_ent_names *const names)
+	const char *const filename)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	mu_ent		*p = NULL;
 	short 		result = 0;
 	unsigned short	track, posn, flags, cvers = 0;
@@ -361,7 +361,7 @@ static mu_ent *file_make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *con
 	/* Actually check for the file in the cache, open it if possible.
 	 * "+ 1" offset to suppress leading '/'.
 	 * Dev note: O_SHLOCK is BSD specific */
-	fdesc = open(names->filename + 1, O_RDONLY|O_NONBLOCK);
+	fdesc = open(filename + 1, O_RDONLY|O_NONBLOCK);
 	if (unlikely(fdesc < 0)) {
 		if (likely((errno == ENOENT) || (errno == EWOULDBLOCK) || (errno == EAGAIN)))
 			return p;	/* Creation of the file is handled separately (playlist.c) */
@@ -441,16 +441,16 @@ error_out:
 /**
  * Creates and writes cache file information.
  *
- * This function creates a new cache file (using mu_ent_names), and
+ * This function creates a new cache file (for #filename), and
  * fills it with the data contained in the mu_ent p structure.
  *
  * @param r Apache request_rec struct to handle log writings.
  * @param p A mu_ent struct containing the data to store.
- * @param conf The config structure used to find out cache configuration.
- * @param names A names structure in which the file name can be found.
+ * @param filename current filename of the file to cache.
  */
-static void cache_file_write(request_rec *r, const mu_ent *const p, const mu_config *const conf, const mu_ent_names * const names)
+static void cache_file_write(request_rec *r, const mu_ent *const p, const char *const filename)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	int	fdesc;
 	FILE	*cache_file = NULL;
 
@@ -462,7 +462,7 @@ static void cache_file_write(request_rec *r, const mu_ent *const p, const mu_con
 		return;
 	
 	/* Dev note: O_EXLOCK is BSD specific. */
-	fdesc = open(names->filename + 1, O_WRONLY|O_NONBLOCK|O_CREAT, S_IRUSR|S_IWUSR);
+	fdesc = open(filename + 1, O_WRONLY|O_NONBLOCK|O_CREAT, S_IRUSR|S_IWUSR);
 	if (fdesc < 0) {
 		if ((errno == EWOULDBLOCK) || (errno == EAGAIN))	/* does this work? */
 			return;	/* Creation of the file is handled separately (playlist.c) */
@@ -518,7 +518,7 @@ int cache_file_setup(cmd_parms *cmd, const char *const setup_string, mu_config *
 
 	if (strncmp(biniou, setup_string, 7) == 0) {
 		ret = -1;
-		const char *restrict csetup = apr_pstrdup(cmd->pool, setup_string+6);
+		char *restrict csetup = apr_pstrdup(cmd->pool, setup_string+6);
 		if (!csetup)
 			goto exit;
 #if 0	/* this is never happening since we check against "file://" */
@@ -539,6 +539,6 @@ int cache_file_setup(cmd_parms *cmd, const char *const setup_string, mu_config *
 	
 exit:
 	if (-1 == ret)
-		mi_serror("Error setting up file cache!");
+		mi_serror("Error setting up %s cache!", "file");
 	return ret;
 }
