@@ -2,7 +2,7 @@
  *  cache-file.h
  *  mod_musicindex
  *
- *  $Id: cache-file.h 658 2006-07-06 14:52:21Z varenet $
+ *  $Id: cache-file.h 705 2007-02-27 17:09:32Z varenet $
  *
  *  Created by Thibaut VARENE on Wed Feb 23 2005.
  *  Copyright (c) 2005 Thibaut VARENE
@@ -16,6 +16,6 @@
 #ifndef CACHE_FILE_H
 #define CACHE_FILE_H
 
-int cache_file_setup(apr_pool_t *pool, const char *const setup_string, mu_config *const conf);
+int cache_file_setup(cmd_parms *cmd, const char *const setup_string, mu_config *const conf);
 
 #endif /* CACHE_FILE_H */
