@@ -2,10 +2,10 @@
  *  cache-mysql.c
  *  mod_musicindex
  *
- *  $Id: cache-mysql.c 982 2012-04-07 10:25:22Z varenet $
+ *  $Id: cache-mysql.c 1011 2012-08-07 20:23:39Z varenet $
  *
  *  Created by Thibaut VARENE on Tue Feb 22 2005.
- *  Copyright (c) 2005,2007,2009-2010 Thibaut VARENE
+ *  Copyright (c) 2005,2007,2009-2010,2012 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -15,12 +15,12 @@
 
 /**
  * @file
- * SQL cache management subsystem.
+ * MySQL cache management subsystem.
  *
  * This cache backend is expected to be impressively efficient. Almost no locking
  * is necessary, since MySQL has its own locking mechanism. Locking is only necessary
  * where things could go wrong if some data were to change between two requests;
- * typically if #mysql_cache_remove_dir() happens between two requests.
+ * typically if mysql_cache_remove_dir() happens between two requests.
  * We try to let MySQL do most of the computation, passing it or retrieving
  * end results for complex queries.
  *
@@ -49,8 +49,8 @@
  * @todo clean error handler when data corruption happens.
  *
  * @author Thibaut Varene
- * @version $Revision: 982 $
- * @date 2005,2007,2009-2010
+ * @version $Revision: 1011 $
+ * @date 2005,2007,2009-2010,2012
  */
 
 #include "playlist.h"
@@ -71,7 +71,7 @@
 #define SQL_SMAX_H	256			/**< max length for host */
 #define SUBSTRINGIFY(x)	STRINGIFY(x)
 #define STRINGIFY(x)	#x
-#define CACHE_VERS	2
+#define CACHE_VERS	3
 #define AINC_OVFLERR	1062			/**< AUTO_INCREMENT overflow error code: 1062 == duplicate entry */
 
 /* http://dev.mysql.com/doc/mysql/en/c.html */
@@ -375,6 +375,10 @@ create:
 		
 	ret = 0;
 	
+	/* optimize table upon server startup. Helps keeping them fresh, and
+	 * is harmless if they were just created */
+	mysql_query(mysql, "OPTIMIZE TABLE `" TABLE_DIRS "`, `" TABLE_FILES "`");
+
 err2:
 	if (mysql_errno(mysql))
 		mi_serror("An error occured: %s", mysql_error(mysql));
@@ -405,12 +409,12 @@ err1:
 static void
 mysql_cache_trunc_tables(request_rec *r, MYSQL *mysql)
 {
-	const char const truncate_tfiles[] = "TRUNCATE TABLE `" TABLE_FILES "`";
-	const char const truncate_tdirs[] = "TRUNCATE TABLE `" TABLE_DIRS "`";
-	
-	if (mysql_query(mysql, truncate_tfiles))
+	if (mysql_query(mysql, "TRUNCATE TABLE `" TABLE_FILES "`"))
 		goto exit;
-	mysql_query(mysql, truncate_tdirs);
+	mysql_query(mysql, "TRUNCATE TABLE `" TABLE_DIRS "`");
+
+	/* since we just did a major change, let's defragment too */
+	mysql_query(mysql, "OPTIMIZE TABLE `" TABLE_DIRS "`, `" TABLE_FILES "`");
 	
 exit:
 	if (mysql_errno(mysql))
@@ -446,7 +450,8 @@ mysql_cache_make_dir(request_rec *r, const char *const dirpath,
 
 	mysql_real_escape_string(mysql, sqlstr, dirpath, strlen(dirpath));
 	
-	/* ON DUPLICATE cannot deal with long paths, we need to do the check by hand:
+	/* ON DUPLICATE cannot deal with long paths (UNIQUE keys must be <1000 bytes),
+	 * we need to do the check by hand:
 	 * SELECT; if (num_rows > 0) UPDATE else INSERT
 	 * Since this is no longer an atomic SQL operation, we have to lock.
 	 * Using LOCK_TABLES as updates shouldn't happen frequently anyway.
@@ -456,7 +461,7 @@ mysql_cache_make_dir(request_rec *r, const char *const dirpath,
 	
 	query = apr_psprintf(r->pool, "SELECT `id` FROM `" TABLE_DIRS "` WHERE `fullpath`='%s'", sqlstr);
 
-	if (mysql_query(mysql, query))
+	if (unlikely(!query) || unlikely(mysql_query(mysql, query)))
 		goto error;
 	
 	mysql_res = mysql_store_result(mysql);
@@ -468,7 +473,7 @@ mysql_cache_make_dir(request_rec *r, const char *const dirpath,
 		query = apr_psprintf(r->pool, "INSERT INTO `" TABLE_DIRS "` (timestamp, fullpath) VALUES ('%lu','%s')",
 				     timestamp, sqlstr);
 			
-	if (mysql_query(mysql, query))
+	if (unlikely(!query) || unlikely(mysql_query(mysql, query)))
 		goto error;
 			
 	ret = CA_OK;
@@ -476,7 +481,7 @@ mysql_cache_make_dir(request_rec *r, const char *const dirpath,
 error:
 	myerrno = mysql_errno(mysql);
 	if (unlikely(myerrno))
-		query = (char *)mysql_error(mysql);
+		query = apr_pstrdup(r->pool, (char *)mysql_error(mysql)); /* static buffer */
 	
 	mysql_query(mysql, "UNLOCK TABLES");
 	
@@ -528,7 +533,7 @@ mysql_cache_remove_dir(request_rec *r, const char *const curdir, MYSQL *mysql)
 	/* If we don't trash the subtree we'll keep junk in the cache */
 	query = apr_psprintf(r->pool, "DELETE FROM `" TABLE_DIRS "` WHERE `fullpath` LIKE '%s%%'", sqlstr);
 
-	if (mysql_query(mysql, query))
+	if (unlikely(!query) || unlikely(mysql_query(mysql, query)))
 		goto error;
 
 	ret = CA_OK;
@@ -553,7 +558,6 @@ error:
  * @warning Asserts path won't contain trailing slashes.
  *
  * @param r Apache request_rec struct to handle log writings.
- * @param conf The config structure used to find out cache configuration.
  * @param path A string containing the full directory path, without trailing '/'.
  * @param mysql mysql handler to communicate with the server.
  *
@@ -564,8 +568,7 @@ error:
  *	CA_FATAL otherwise.
  */
 static int
-mysql_cache_check_dir(request_rec *r, const mu_config *const conf,
-			const char *const path, MYSQL *mysql)
+mysql_cache_check_dir(request_rec *r, const char *const path, MYSQL *mysql)
 {
 	MYSQL_RES	*mysql_res = NULL;
 	MYSQL_ROW	mysql_row;
@@ -573,10 +576,10 @@ mysql_cache_check_dir(request_rec *r, const mu_config *const conf,
 	char		*restrict sqlstr = NULL, *restrict query = NULL;
 	struct stat	dirstat;
 
-	if (!path)
+	if (unlikely(!path))
 		return ret;
 
-	if (!(sqlstr = malloc(2*strlen(path)+1)))
+	if (unlikely(!(sqlstr = malloc(2*strlen(path)+1))))
 		goto error;
 	
 	mysql_real_escape_string(mysql, sqlstr, path, strlen(path));
@@ -585,7 +588,7 @@ mysql_cache_check_dir(request_rec *r, const mu_config *const conf,
 	query = apr_psprintf(r->pool, "SELECT timestamp FROM `" TABLE_DIRS "` WHERE `fullpath`='%s'", sqlstr);
 	free(sqlstr);
 
-	if (mysql_query(mysql, query))
+	if (unlikely(!query) || unlikely(mysql_query(mysql, query)))
 		goto error;
 
 	mysql_res = mysql_store_result(mysql);
@@ -597,7 +600,7 @@ mysql_cache_check_dir(request_rec *r, const mu_config *const conf,
 		unsigned long	test;
 		mysql_row = mysql_fetch_row(mysql_res);
 		
-		if (!mysql_row)
+		if (unlikely(!mysql_row))
 			goto error;
 
 		ret = CA_OK;
@@ -635,16 +638,16 @@ error:
  * @param r Apache request_rec struct to handle log writings.
  * @param pool Apache pool
  * @param in Not used (except for closing).
- * @param conf MusicIndex configuration paramaters struct.
- * @param names struct names to get the current filename.
+ * @param filename current filename.
  *
  * @return When possible, struct mu_ent correctly set up, file stream closed,
  * 		NULL otherwise.
  */
 static mu_ent*
 mysql_cache_make_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	const mu_config *const conf, mu_ent_names *const names)
+	const char *const filename)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	MYSQL		*mysql = conf->cache_setup;
 	MYSQL_RES	*mysql_res = NULL;
 	MYSQL_ROW	mysql_row;
@@ -656,11 +659,11 @@ mysql_cache_make_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	if (unlikely((!mysql) || (!mysql_params.ready)))
 		return p;
 	
-	if (unlikely(stat(names->filename, &statbuf)))
+	if (unlikely(stat(filename, &statbuf)))
 		return p;
 
-	dirs = strdup(names->filename);
-	bases = strdup(names->filename);
+	dirs = strdup(filename);
+	bases = strdup(filename);
 	if (unlikely(!dirs || !bases))
 		goto exit;
 
@@ -681,7 +684,7 @@ mysql_cache_make_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 			     "AND `filename`='%s' COLLATE utf8_bin",
 			     sqldirn, sqlbasen);
 	
-	if (unlikely(mysql_query(mysql, query)))
+	if (unlikely(!query) || unlikely(mysql_query(mysql, query)))
 		goto error;
 
 	mysql_res = mysql_store_result(mysql);
@@ -702,7 +705,7 @@ mysql_cache_make_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 		goto exit;
 
 	p = mysql_cache_new_ent(pool, mysql_row);
-	if (p)
+	if (likely(p))
 		fclose(in);	/* mandatory */
 
 error:
@@ -718,20 +721,19 @@ exit:
 /**
  * Creates and writes cache information.
  *
- * This function creates a new cache entry for (names->filename), and
+ * This function creates a new cache entry for (using #filename), and
  * fills it with the data contained in the mu_ent p structure.
  *
  * @bug we don't check enough return values of apr_psprintf()/apr_pstrcat()...
  *
  * @param r Apache request_rec struct to handle log writings and pool allocation.
  * @param p A mu_ent struct containing the data to store.
- * @param conf The config structure used to find out cache configuration.
- * @param names A names structure in which the file name can be found.
+ * @param filename current filename of the file to cache.
  */
 static void
-mysql_cache_write(request_rec *r, const mu_ent *const p,
-	const mu_config *const conf, const mu_ent_names *const names)
+mysql_cache_write(request_rec *r, const mu_ent *const p, const char *const filename)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	MYSQL		*mysql = conf->cache_setup;
 	MYSQL_RES	*mysql_res = NULL;
 	char		* restrict sqldirn = NULL, * restrict sqlbasen = NULL, * restrict query = NULL;
@@ -748,14 +750,14 @@ mysql_cache_write(request_rec *r, const mu_ent *const p,
 	 * mysql_cache_dircontents() will not try to return its content later on. We
 	 * thus "invalidate" this cache entry by giving it a 0 timestamp */
 	if (p->filetype < 0) {
-		int check = mysql_cache_check_dir(r, conf, names->filename, mysql);
+		int check = mysql_cache_check_dir(r, filename, mysql);
 		if (CA_CREATE == check)
-			mysql_cache_make_dir(r, names->filename, 0, mysql);
+			mysql_cache_make_dir(r, filename, 0, mysql);
 		goto exit;
 	}
 	
-	dirs = strdup(names->filename);
-	bases = strdup(names->filename);
+	dirs = strdup(filename);
+	bases = strdup(filename);
 	if (unlikely(!dirs || !bases))
 		goto exit;
 	
@@ -765,7 +767,7 @@ mysql_cache_write(request_rec *r, const mu_ent *const p,
 	sqldirn = malloc(2*strlen(dirn)+1);
 	sqlbasen = malloc(2*strlen(basen)+1);
 	if (unlikely(!sqldirn || !sqlbasen))
-		goto error;
+		goto exit;
 
 	mysql_real_escape_string(mysql, sqldirn, dirn, strlen(dirn));
 	mysql_real_escape_string(mysql, sqlbasen, basen, strlen(basen));
@@ -796,7 +798,7 @@ mysql_cache_write(request_rec *r, const mu_ent *const p,
 	
 	query = apr_psprintf(r->pool, "SELECT `id` FROM `" TABLE_DIRS "` WHERE `fullpath`='%s'", sqldirn);
 	
-	if (unlikely(mysql_query(mysql, query)))
+	if (unlikely(!query) || unlikely(mysql_query(mysql, query)))
 		goto error;
 	
 	mysql_res = mysql_store_result(mysql);
@@ -813,7 +815,7 @@ mysql_cache_write(request_rec *r, const mu_ent *const p,
 			     "AND pid=(SELECT `id` FROM `" TABLE_DIRS "` WHERE `fullpath`='%s')",
 			     sqlbasen, sqldirn);
 
-	if (unlikely(mysql_query(mysql, query)))
+	if (unlikely(!query) || unlikely(mysql_query(mysql, query)))
 		goto error;
 	
 	mysql_res = mysql_store_result(mysql);
@@ -829,50 +831,42 @@ mysql_cache_write(request_rec *r, const mu_ent *const p,
 			"` SET cvers='%hi',filetype='%hu',flags='%hu',track='%hu',"
 			"posn='%hu',date='%hu',freq='%hu',length='%hu',"
 			"bitrate='%lu',size='%lu',mtime='%lu',title='%s'",
-			CACHE_VERS, p->filetype, p->flags, p->track, p->posn, p->date,
+			CACHE_VERS, p->filetype, (p->flags & EF_FLAGSTOSAVE), p->track, p->posn, p->date,
 			p->freq, p->length, p->bitrate, p->size, p->mtime, sqltit);
-		if (sqlalb)
-			query = apr_pstrcat(r->pool, query, apr_psprintf(r->pool, ",album='%s'", sqlalb), NULL);
-		else
-			query = apr_pstrcat(r->pool, query, ",album=NULL", NULL);
-		if (sqlart)
-			query = apr_pstrcat(r->pool, query, apr_psprintf(r->pool, ",artist='%s'", sqlart), NULL);
-		else
-			query = apr_pstrcat(r->pool, query, ",artist=NULL", NULL);
-		if (sqlgen)
-			query = apr_pstrcat(r->pool, query, apr_psprintf(r->pool, ",genre='%s'", sqlgen), NULL);
-		else
-			query = apr_pstrcat(r->pool, query, ",genre=NULL", NULL);
-		query = apr_pstrcat(r->pool, query,  apr_psprintf(r->pool, " WHERE id='%lu'", (unsigned long)atol(mysql_row[0])), NULL);
+		
+		query = apr_pstrcat(r->pool, query,	/* XXX no alloc error checking on psprintf() */
+				sqlalb ? apr_psprintf(r->pool, ",album='%s'", sqlalb) : ",album=NULL",
+				sqlart ? apr_psprintf(r->pool, ",artist='%s'", sqlart) : ",artist=NULL",
+				sqlgen ? apr_psprintf(r->pool, ",genre='%s'", sqlgen) : ",genre=NULL",
+				apr_psprintf(r->pool, " WHERE id='%lu'", (unsigned long)atol(mysql_row[0])), NULL);
 	}
 	else {
-		char *query2;
 		query = apr_psprintf(r->pool, "INSERT INTO `" TABLE_FILES
 			"` (`pid`,`cvers`,`filetype`,`flags`,`track`,`posn`,`date`,`freq`,"
-			"`length`,`bitrate`,`size`,`mtime`,`album`,`artist`,`genre`,`title`,"
-			"`filename`) VALUES "
+			"`length`,`bitrate`,`size`,`mtime`,`title`,`filename`,`album`,`artist`,"
+			"`genre`) VALUES "
 			"((SELECT `id` FROM `" TABLE_DIRS "` WHERE `fullpath`='%s'),"
 			"'%hi','%hu','%hu','%hu','%hu','%hu','%hu','%hu',"
-			"'%lu','%lu','%lu',",
-			sqldirn, CACHE_VERS, p->filetype, p->flags, p->track, p->posn, p->date,
-			p->freq, p->length, p->bitrate, p->size, p->mtime);
-		query2 = sqlalb ? apr_psprintf(r->pool, "'%s',", sqlalb) : "NULL,";
-		query = apr_pstrcat(r->pool, query, query2, NULL);
-		query2 = sqlart ? apr_psprintf(r->pool, "'%s',", sqlart) : "NULL,";
-		query = apr_pstrcat(r->pool, query, query2, NULL);
-		query2 = sqlgen ? apr_psprintf(r->pool, "'%s',", sqlgen) : "NULL,";
-		query = apr_pstrcat(r->pool, query, query2, NULL);
-		query2 = apr_psprintf(r->pool, "'%s','%s')", sqltit, sqlbasen);
-		query = apr_pstrcat(r->pool, query, query2, NULL);
+			"'%lu','%lu','%lu','%s','%s',",
+			sqldirn, CACHE_VERS, p->filetype, (p->flags & EF_FLAGSTOSAVE), p->track, p->posn, p->date,
+			p->freq, p->length, p->bitrate, p->size, p->mtime, sqltit, sqlbasen);
+		query = apr_pstrcat(r->pool, query,
+				sqlalb ? apr_psprintf(r->pool, "'%s',", sqlalb) : "NULL,", /* XXX no alloc error checking on psprintf() */
+				sqlart ? apr_psprintf(r->pool, "'%s',", sqlart) : "NULL,",
+				sqlgen ? apr_psprintf(r->pool, "'%s'", sqlgen) : "NULL",
+				")", NULL);
 	}
 
 	/* submit the query */
-	mysql_query(mysql, query);
+	if (likely(query))
+		mysql_query(mysql, query);
 
 error:
 	myerrno = mysql_errno(mysql);
-	if (unlikely(myerrno))
-		query = (char *)mysql_error(mysql);
+	if (unlikely(myerrno)) {
+		query = apr_pstrdup(r->pool, (char *)mysql_error(mysql)); /* have to dup as mysql_error() sends a static buffer */
+		mysql_cache_make_dir(r, dirn, 0, mysql); /* the query failed we must invalidate the directory. XXX no fail check */
+	}
 	
 	mysql_query(mysql, "UNLOCK TABLES");
 	
@@ -908,13 +902,13 @@ typedef struct mysql_dir {
  *	     cache directory if they're gone or b) requeue them if they've changed.
  *
  * @warning Asserts we're already connected to the db.
- * @warning Asserts names->filename doesn't contain trailing /.
+ * @warning Asserts filename doesn't contain trailing /.
  *
  * @param r Apache request_rec struct to handle log writings.
  * @param pool Pool allocation.
  * @param pack A pack to add new entries to.
- * @param conf The config structure used to find out cache configuration.
- * @param names A names structure in which the current path can be found.
+ * @param filename current file path.
+ * @param uri current server-side uri.
  * @param mysql mysql handler to talk to the server.
  * @param soptions Flags to use for created entries.
  *
@@ -923,9 +917,9 @@ typedef struct mysql_dir {
  */
 static mysql_dir
 *mysql_cache_dircontents(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
-	const mu_config *const conf, const mu_ent_names *const names, MYSQL *mysql,
-	unsigned long soptions)
+	const char *const filename, const char *const uri, MYSQL *mysql, unsigned long soptions)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	MYSQL_RES	*mysql_res;
 	MYSQL_ROW	mysql_row;
 	mu_ent		*p = NULL;
@@ -934,7 +928,7 @@ static mysql_dir
 	char		*restrict sqlstr = NULL, *restrict query = NULL;
 	char		*restrict basen = NULL, *restrict fullpath = NULL;
 	unsigned long	nb = 0, size = 0;
-	const size_t	fnlen = strlen(names->filename);
+	const size_t	fnlen = strlen(filename);
 	
 	/* 1. we need to treat *only* the immediate subdirs, as we can't assert that treating all
 	subdirs is idempotent (eg: won't duplicate work with the rest of make_music_entry()).
@@ -944,7 +938,7 @@ static mysql_dir
 	if (unlikely(!sqlstr))
 		goto error;
 
-	mysql_real_escape_string(mysql, sqlstr, names->filename, fnlen);
+	mysql_real_escape_string(mysql, sqlstr, filename, fnlen);
 
 	/* this request should hopefully return only immediate subdirs,
 	as it will only match entries not containing any '/' after the current
@@ -956,18 +950,20 @@ static mysql_dir
 				"`fullpath` LIKE '%s/%%' AND (LOCATE('/', fullpath, %lu)=0)",
 						 sqlstr, strlen(sqlstr)+2);
 
-	if (unlikely(mysql_query(mysql, query)))
+	if (unlikely(!query) || unlikely(mysql_query(mysql, query)))
 		goto error;
 
 	mysql_res = mysql_use_result(mysql);
 	
 	while ((mysql_row = mysql_fetch_row(mysql_res))) {
-			basen = apr_pstrdup(pool, mysql_row[0]+(fnlen+1));
-			/* simulate basename by skipping the prefix path */
-			mdir = (mysql_dir *)apr_pcalloc(pool, sizeof(mysql_dir));
+		basen = apr_pstrdup(pool, mysql_row[0]+(fnlen+1));
+		/* simulate basename by skipping the prefix path */
+		mdir = (mysql_dir *)apr_pcalloc(pool, sizeof(mysql_dir));
+		if (likely(basen && mdir)) {
 			mdir->d_name = basen;
 			mdir->next = mhead;
 			mhead = mdir;
+		}
 	}
 
 	mysql_free_result(mysql_res);
@@ -993,18 +989,18 @@ static mysql_dir
 							escsearch, "')", NULL);
 	}
 	
-	if (unlikely(mysql_query(mysql, query)))
+	if (unlikely(!query) || unlikely(mysql_query(mysql, query)))
 		goto error;
 
 	mysql_res = mysql_use_result(mysql);
 	
-	/* names->filename + '/' + '\0' */
+	/* filename + '/' + '\0' */
 	fullpath = malloc(fnlen + 2);
 	if (!fullpath) {
 		ret = NULL;
 		goto error;
 	}
-	strncpy(fullpath, names->filename, fnlen);
+	strncpy(fullpath, filename, fnlen);
 	fullpath[fnlen] = '/';
 	fullpath[fnlen+1] = '\0';
 	
@@ -1014,9 +1010,11 @@ static mysql_dir
 			/* XXX MEM OPTIM: malloc here, free in readdir() */
 			basen = apr_pstrdup(pool, mysql_row[SQL_CDATAN]);
 			mdir = (mysql_dir *)apr_pcalloc(pool, sizeof(mysql_dir));
-			mdir->d_name = basen;
-			mdir->next = mhead;
-			mhead = mdir;
+			if (likely(basen && mdir)) {
+				mdir->d_name = basen;
+				mdir->next = mhead;
+				mhead = mdir;
+			}
 		}
 		else {
 			p = mysql_cache_new_ent(pool, mysql_row);
@@ -1035,9 +1033,9 @@ static mysql_dir
 				 * If 1) is not verified, we trash the directory cache and exit
 				 * If 2) is not verified, we requeue the file */
 				if (unlikely(stat(fullpath, &statbuf))) {
-					mi_rdebug("Trashing cache for: %s", names->filename);
+					mi_rdebug("Trashing cache for: %s", filename);
 					mysql_free_result(mysql_res);
-					mysql_cache_remove_dir(r, names->filename, mysql);
+					mysql_cache_remove_dir(r, filename, mysql);
 					ret = NULL;
 					goto error;
 				}
@@ -1045,13 +1043,15 @@ static mysql_dir
 					mi_rdebug("Requeuing: %s", fullpath);
 					basen = apr_pstrdup(pool, mysql_row[SQL_CDATAN]);
 					mdir = (mysql_dir *)apr_pcalloc(pool, sizeof(mysql_dir));
-					mdir->d_name = basen;
-					mdir->next = mhead;
-					mhead = mdir;
+					if (likely(basen && mdir)) {
+						mdir->d_name = basen;
+						mdir->next = mhead;
+						mhead = mdir;
+					}
 					continue;
 				}
 				
-				p->uri = apr_pstrcat(pool, names->uri, mysql_row[SQL_CDATAN], NULL);
+				p->uri = apr_pstrcat(pool, uri, mysql_row[SQL_CDATAN], NULL);
 				p->filename = p->file = p->uri;
 				p->filename = (strrchr(p->filename, '/') + 1);
 				P_FLAGS_OPTIONS(p, soptions);
@@ -1098,24 +1098,24 @@ error:
  * Cache up-to-dateness management is done here, as directory creation can happen
  * elsewhere in the code (in particular in mysql_cache_make_entry()).
  *
- * @warning Asserts @a names->filename won't contain trailing /.
+ * @warning Asserts @a filename won't contain trailing /.
  * @todo uses MI_QUICKPL, API should be fixed.
  *
  * @param r Apache request_rec struct to handle log writings and pool allocation.
  * @param pack A pack to add new entries to.
- * @param conf The config structure used to find out cache configuration.
- * @param names A names structure in which the current path can be found.
+ * @param filename current file path.
+ * @param uri current server-side uri.
  * @param soptions Flags to use for created entries.
  *
  * @return the chained list from mysql_cache_dircontents(), if any, NULL otherwise.
  */
 static void
-*mysql_cache_opendir(request_rec *r, mu_pack *const pack, const mu_config *const conf,
-			const mu_ent_names * const names, unsigned long soptions)
+*mysql_cache_opendir(request_rec *r, mu_pack *const pack, const char * const filename,
+			const char * const uri, unsigned long soptions)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	MYSQL		*mysql = conf->cache_setup;
 	int		val;
-	const char *const path = names->filename;
 	mysql_dir	*mdir = NULL;
 	void		*ret = NULL;
 	struct stat	dirstat;
@@ -1125,28 +1125,28 @@ static void
 		goto error;
 	}
 
-	val = mysql_cache_check_dir(r, conf, path, mysql);
+	val = mysql_cache_check_dir(r, filename, mysql);
 	switch (val) {
 		case CA_OK:
 			break;
 		case CA_STALE:
-			mysql_cache_remove_dir(r, path, mysql);
+			mysql_cache_remove_dir(r, filename, mysql);
 		case CA_CREATE:
 		case CA_NOTREADY:
-			if (!(conf->options & MI_QUICKPL)) {
+			if (likely(!(conf->options & MI_QUICKPL))) {
 				/* XXX HACK. This is a gross workaround until the interface
 				 * is better specified: we don't want to record a directory
 				 * in the cache as valid when we're not going to record any
 				 * content for it, which happens when MI_QUICKPL is on */
-				stat(path, &dirstat);
-				mysql_cache_make_dir(r,	path, dirstat.st_mtime, mysql);
+				stat(filename, &dirstat);
+				mysql_cache_make_dir(r,	filename, dirstat.st_mtime, mysql);
 			}
 		case CA_FATAL:
 		default:
 			goto error;
 	}
 	
-	mdir = mysql_cache_dircontents(r, r->pool, pack, conf, names, mysql, soptions);
+	mdir = mysql_cache_dircontents(r, r->pool, pack, filename, uri, mysql, soptions);
 	
 	ret = (void *)mdir;	/* NULL if mysql_cache_dircontents() failed, which is what we want */
 
@@ -1158,10 +1158,10 @@ error:
  * Returns directory contents to be processed in make_music_entry() a la readdir().
  *
  * @param dir pointer to the head of the chainlist of elements
- * @param conf module configuration (ignored)
+ * @return filename
  */
 static const char
-*mysql_cache_readdir(void *dir, const mu_config *const conf)
+*mysql_cache_readdir(void *dir)
 {
 	mysql_dir	*mdir;
 	const char	*ret = NULL;
@@ -1186,11 +1186,11 @@ static const char
  * We really only want one database connection max per request.
  *
  * @param r Apache request_rec struct to handle log writings and pool allocation.
- * @param conf The config structure used to store cache private data.
  */
 static void
-mysql_cache_prologue(request_rec *r, mu_config *const conf)
+mysql_cache_prologue(request_rec *r)
 {
+	mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	MYSQL	*mysql = NULL;
 
 	if (unlikely(!mysql_params.ready))
@@ -1223,11 +1223,11 @@ out:
  * database connection that we established there.
  *
  * @param r Apache request_rec struct to handle log writings and pool allocation.
- * @param conf The config structure used to store cache private data.
  */
 static void
-mysql_cache_epilogue(request_rec *r, mu_config *const conf)
+mysql_cache_epilogue(request_rec *r)
 {
+	mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	MYSQL	*mysql = (MYSQL *)conf->cache_setup;
 	
 	if (unlikely(!mysql))
