@@ -2,11 +2,11 @@
  *  config.c
  *  mod_musicindex
  *
- *  $Id: config.c 962 2011-10-31 17:32:29Z varenet $
+ *  $Id: config.c 1001 2012-07-30 18:45:20Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
- *  Copyright (c) 2003-2005,2007,2009 Thibaut VARENE
+ *  Copyright (c) 2003-2005,2007,2009,2012 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -20,15 +20,17 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 962 $
+ * @version $Revision: 1001 $
  * @date 2003-2007
  * @date 2009
+ * @date 2012
  *
  * That file takes care of the module configuration, either via setting some
  * known default values, and/or by getting them from environmental configuration
  * variables passed to apache in its config file.
  *
- * http://www.apachetutor.org/dev/config
+ * http://www.apachetutor.org/dev/config \n
+ * http://www.kuzbass.ru/docs/apache_c_mod_perl/147.htm
  */
 
 #include "config.h"
@@ -95,8 +97,11 @@ static const char const default_css[] = "musicindex.css";	/**< Default CSS file
  * This function creates a new configuration structure and fills it with
  * default values.
  *
+ * @note It kinda bugs me that we don't need to set @a options_not = ~0, but it
+ * seems to work without it...
+ *
  * @param p Apache pool for memory allocation.
- * @param dummy String given by apache we don't use.
+ * @param dummy string containing the path to the relevant Directory or Location - we don't use it.
  *
  * @return The newly created configuration structure.
  */
@@ -104,28 +109,14 @@ void *create_musicindex_config(apr_pool_t *p, char *dummy)
 {
 	mu_config *const new = (mu_config *)apr_pcalloc(p, sizeof(mu_config));
 
-	memset(new->order, 0, sizeof(new->order));
-	memset(new->fields, 0, sizeof(new->fields));
-
 	memcpy(new->fields, default_fields, sizeof(default_fields));
 	memcpy(new->order, default_order, sizeof(default_order));
 	
 	new->title = default_rootname;
-
 	new->directory = default_directory;
 	new->css = default_css;
-	new->search = NULL;
-	new->iceserver = NULL;
-
-	new->cache = NULL;
-	new->cache_setup = NULL;
 
 	new->cookie_life = CONF_COOKIE_LIFE;
-	new->rss_items = 0;
-
-	new->options = 0;
-	new->options_not = 0;
-
 	new->dir_per_line = CONF_DIRPERLINE;
 
 	return (void *)new;
@@ -138,26 +129,28 @@ void *create_musicindex_config(apr_pool_t *p, char *dummy)
  * apply a simple inheritance pattern from the parent directory(ies).
  *
  * @note In the absence of a .htaccess in the hierarchy of folders being accessed,
- * @a basev contains a freshly created config structure (through
- * create_musicindex_config()) and @a addv contains another config struct created
+ * #basev contains a freshly created config structure (through
+ * create_musicindex_config()) and #addv contains another config struct created
  * through create_musicindex_config(), with the configuration parameters modified
- * by <Directory> directive (if any) set appropriately.
+ * by \<Directory\> directive (if any) set appropriately.\n
+ *	#basev: clean	#addv: configured	=> keep #addv
  *
  * @note In the presence of a .htaccess file in the directory being accessed,
- * @a basev contains the result of the merging of all "parent" configuration directives
- * (<Directory> being the top ancestor, and any subsquent .htaccess being merged
+ * #basev contains the result of the merging of all "parent" configuration directives
+ * (\<Directory\> being the top ancestor, and any subsquent .htaccess being merged
  * using merge_musicindex_configs()) up to the one in the current directory being
- * accessed, which config options are set in @a addv.
+ * accessed, which config options are set in #addv.\n
+ *	#basev: merged	#addv: configured	=> merge
  *
  * @note In the presence of a .htaccess anywhere in the parent hierarchy of the
- * directory being accessed @b BUT in said directory, @a basev contains a pristine
- * config structure (as per create_musicindex_config()) and @a addv contains the
- * result of merging all parent configuration directives (<Directory> being the
- * top ancestor).
+ * directory being accessed @b BUT in said directory, #basev contains a pristine
+ * config structure (as per create_musicindex_config()) and #addv contains the
+ * result of merging all parent configuration directives (\<Directory\> being the
+ * top ancestor).\n
+ *	#basev: clean	#addv: merged		=> keep #addv
  *
- * Hence the logic of this function: if @a addv contains init data (data setup in
- * create_musicindex_config()), use @a basev element. Otherwise, @a addv contains
- * modified config directive, use it instead.
+ * Hence the logic of this function: if #basev contains init data (data setup in
+ * create_musicindex_config()), use #addv element. Otherwise, merge both configs.
  *
  * @param p Apache pool for memory allocation.
  * @param basev Pointer to main configuration structure.
@@ -171,41 +164,46 @@ void *merge_musicindex_configs(apr_pool_t *p, void *basev, void *addv)
 	mu_config *const restrict base = (mu_config *)basev;
 	mu_config *const restrict add = (mu_config *)addv;
 
-	if (!memcmp(add->order, default_order, sizeof(default_order)))
-		memcpy(new->order, base->order, sizeof(base->order));
-	else
+	if (!base->sets) {
+		/* basev is empty we can ignore it */
 		memcpy(new->order, add->order, sizeof(add->order));
-
-	if (!memcmp(add->fields, default_fields, sizeof(default_fields)))
-		memcpy(new->fields, base->fields, sizeof(base->fields));
-	else
 		memcpy(new->fields, add->fields, sizeof(add->fields));
-
-	if (add->title == default_rootname)
-		new->title = base->title;
-	else
 		new->title = add->title;
-
-	/* directory is currently non-modifiable */
-	new->directory = default_directory;
-
-	if (add->css == default_css)
-		new->css = base->css;
-	else
 		new->css = add->css;
-
-	if (add->cache == NULL) {
-		new->cache = base->cache;
-		new->cache_setup = base->cache_setup;
-	} else {
+		new->iceserver = add->iceserver;
+		new->cookie_life = add->cookie_life;
+		new->rss_items = add->rss_items;
+		new->dir_per_line = add->dir_per_line;
 		new->cache = add->cache;
 		new->cache_setup = add->cache_setup;
 	}
+	else {
+		/* we need to merge the configurations */
+		if (add->sets & CF_ORDER)
+			memcpy(new->order, add->order, sizeof(add->order));
+		else
+			memcpy(new->order, base->order, sizeof(base->order));
 
-	if (add->iceserver == NULL)
-		new->iceserver = base->iceserver;
-	else
-		new->iceserver = add->iceserver;
+		if (add->sets & CF_FIELDS)
+			memcpy(new->fields, add->fields, sizeof(add->fields));
+		else
+			memcpy(new->fields, base->fields, sizeof(base->fields));
+
+		new->title = (add->sets & CF_TITLE) ? add->title : base->title;
+		new->css = (add->sets & CF_CSS) ? add->css : base->css;
+		new->iceserver = (add->sets & CF_ICES) ? add->iceserver : base->iceserver;
+		new->cookie_life = (add->sets & CF_CKLF) ? add->cookie_life : base->cookie_life;
+		new->rss_items = (add->sets & CF_RSS) ? add->rss_items : base->rss_items;
+		new->dir_per_line = (add->sets & CF_DPL) ? add->dir_per_line : base->dir_per_line;
+
+		if (add->sets & CF_CACHE) {
+			new->cache = add->cache;
+			new->cache_setup = add->cache_setup;
+		} else {
+			new->cache = base->cache;
+			new->cache_setup = base->cache_setup;
+		}
+	}
 
 	new->options = base->options | add->options;
 	new->options &= ~(base->options_not | add->options_not);
@@ -213,20 +211,11 @@ void *merge_musicindex_configs(apr_pool_t *p, void *basev, void *addv)
 	new->options_not = base->options_not | add->options_not;
 	new->options_not &= ~(base->options | add->options);
 
-	if (add->cookie_life == CONF_COOKIE_LIFE)
-		new->cookie_life = base->cookie_life;
-	else
-		new->cookie_life = add->cookie_life;
+	/* directory is currently non-modifiable */
+	new->directory = default_directory;
 
-	if (add->rss_items == 0)
-		new->rss_items = base->rss_items;
-	else
-		new->rss_items = add->rss_items;
-
-	if (add->dir_per_line == CONF_DIRPERLINE)
-		new->dir_per_line = base->dir_per_line;
-	else
-		new->dir_per_line = add->dir_per_line;
+	/* flag new as containing meaningful data */
+	new->sets |= CF_MERGED;
 
 	return new;
 }
@@ -276,12 +265,15 @@ static const char *sort_order(cmd_parms *cmd, void *d, const char *optstr)
 	mu_config *const d_cfg = (mu_config *)d;
 	unsigned short i = sort_or_fields(cmd, d_cfg->order, optstr);
 	d_cfg->order[i] = SB_URI;	/* fallback on URI sort */
+	d_cfg->sets |= CF_ORDER;
 	return NULL;
 }
 
 static const char *set_fields(cmd_parms *cmd, void *d, const char *optstr)
 {
-	sort_or_fields(cmd, ((mu_config *)d)->fields, optstr);
+	mu_config *const d_cfg = (mu_config *)d;
+	sort_or_fields(cmd, d_cfg->fields, optstr);
+	d_cfg->sets |= CF_FIELDS;
 	return NULL;
 }
 
@@ -367,11 +359,13 @@ static const char *basic_config(cmd_parms *cmd, void *d, const char *optstr)
  */
 static const char *set_cache_uri(cmd_parms *cmd, void *d, const char *optstr)
 {
+	mu_config *const d_cfg = (mu_config *)d;
 	int i, ret = 1;
 
 	for (i = 0; (ret != 0) && (cache_setups[i] != NULL); i++)
-		ret = cache_setups[i](cmd, optstr, (mu_config *)d);
+		ret = cache_setups[i](cmd, optstr, d_cfg);
 
+	d_cfg->sets |= CF_CACHE;
 	return NULL;
 }
 
@@ -388,11 +382,12 @@ static const char *set_cache_uri(cmd_parms *cmd, void *d, const char *optstr)
  */
 static const char *set_page_title(cmd_parms *cmd, void *d, const char *optstr)
 {
-	mu_config *const cfg = (mu_config *)d;
+	mu_config *const d_cfg = (mu_config *)d;
 	if ((optstr == NULL) || (optstr[0] == '\0'))
-		cfg->title = NULL;
+		d_cfg->title = NULL;
 	else
-		cfg->title = apr_pstrdup(cmd->pool, optstr);
+		d_cfg->title = apr_pstrdup(cmd->pool, optstr);
+	d_cfg->sets |= CF_TITLE;
 	return NULL;
 }
 
@@ -408,8 +403,9 @@ static const char *set_page_title(cmd_parms *cmd, void *d, const char *optstr)
  */
 static const char *set_ice_server(cmd_parms *cmd, void *d, const char *optstr)
 {
-	mu_config *const cfg = (mu_config *)d;
-	cfg->iceserver = apr_pstrdup(cmd->pool, optstr);
+	mu_config *const d_cfg = (mu_config *)d;
+	d_cfg->iceserver = apr_pstrdup(cmd->pool, optstr);
+	d_cfg->sets |= CF_ICES;
 	return NULL;
 }
 
@@ -425,7 +421,9 @@ static const char *set_ice_server(cmd_parms *cmd, void *d, const char *optstr)
  */
 static const char *set_css_default(cmd_parms *cmd, void *d, const char *optstr)
 {
-	((mu_config *)d)->css = apr_pstrdup(cmd->pool, optstr);
+	mu_config *const d_cfg = (mu_config *)d;
+	d_cfg->css = apr_pstrdup(cmd->pool, optstr);
+	d_cfg->sets |= CF_CSS;
 	return NULL;
 }
 
@@ -442,7 +440,9 @@ static const char *set_css_default(cmd_parms *cmd, void *d, const char *optstr)
  */
 static const char *set_cookie_life(cmd_parms *cmd, void *d, const char *optstr)
 {
-	((mu_config *)d)->cookie_life = atoi(optstr);
+	mu_config *const d_cfg = (mu_config *)d;
+	d_cfg->cookie_life = atoi(optstr);
+	d_cfg->sets |= CF_CKLF;
 	return NULL;
 }
 
@@ -458,13 +458,17 @@ static const char *set_cookie_life(cmd_parms *cmd, void *d, const char *optstr)
  */
 static const char *set_dirperline(cmd_parms *cmd, void *d, const char *optstr)
 {
-	((mu_config *)d)->dir_per_line = (atoi(optstr) ? atoi(optstr) : CONF_DIRPERLINE);
+	mu_config *const d_cfg = (mu_config *)d;
+	d_cfg->dir_per_line = (atoi(optstr) ? atoi(optstr) : CONF_DIRPERLINE);
+	d_cfg->sets |= CF_DPL;
 	return NULL;
 }
 
 /**
  * Sets the default display.
  *
+ * @bug Wouldn't this override whatever is set with MusicSortOrder?
+ *
  * @param cmd Struct containing a pointer to the pool I have to use.
  * @param d Pointer to configuration structure.
  * @param optstr The string given as parameter in the configuration.
@@ -476,15 +480,17 @@ static const char *set_display(cmd_parms *cmd, void *d, const char *optstr)
 	mu_config *const cfg = (mu_config *)d;
 	if (strcmp(optstr, "RSS") == 0) {
 		cfg->options |= MI_RSS;
+		cfg->options &= ~MI_RECURSIVE;
 		cfg->order[0] = SB_MTIME;
 		cfg->order[1] = SB_URI;
-		cfg->options &= ~MI_RECURSIVE;
-		cfg->rss_items = CONF_RSS_ITEMS;
+		cfg->rss_items = CONF_RSS_ITEMS; /* XXX necessary? */
+		cfg->sets |= (CF_ORDER | CF_RSS);
 	}
 	else if (strcmp(optstr, "HTML") == 0) {
 		cfg->options &= ~MI_RSS;
 		cfg->options_not |= MI_RSS;
-		memcpy(cfg->order, default_order, sizeof(default_order));
+		memcpy(cfg->order, default_order, sizeof(default_order)); /* XXX necessary? */
+		/* cfg->sets |= CF_ORDER; */
 	}
 	return NULL;
 }
