@@ -2,7 +2,7 @@
  *  http.c
  *  mod_musicindex
  *
- *  $Id: http.c 823 2009-02-05 19:38:14Z varenet $
+ *  $Id: http.c 1003 2012-07-31 09:53:17Z varenet $
  *
  *  Created by Regis BOUDIN on Sun Jun 13 2004.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -20,7 +20,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 823 $
+ * @version $Revision: 1003 $
  * @date 2004-2007
  *
  * This file is here to deal with HTTP related
@@ -71,10 +71,10 @@
  * and sets the handler options accordingly.
  * 
  * @param r Apache request_rec struct to handle connection details.
- * @param conf MusicIndex configuration paramaters struct.
  */
-void treat_get_args(request_rec *r, mu_config *const conf)
+void treat_get_args(request_rec *r)
 {
+	mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	const char *s = r->args;
 	const char *p;
 	register unsigned short i;
@@ -166,10 +166,10 @@ void treat_get_args(request_rec *r, mu_config *const conf)
  * but it's pretty ugly as well.
  *
  * @param r Apache request_rec struct to handle connection details.
- * @param conf MusicIndex configuration paramaters struct.
  */
-void treat_post_args(request_rec *r, mu_config *const conf)
+void treat_post_args(request_rec *r)
 {
+	mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	char buffer[MAX_STRING];
 	const char *s = NULL;
 	const char *p = s;
@@ -330,11 +330,11 @@ static const char *find_or_create_playlist(request_rec *r, apr_pool_t *subpool)
  *
  * @param r the apache request
  * @param subpool the current subpool to use for mem allocations
- * @param conf the current configuration flags (including the post arguments)
  * @return a string to store in the cookie
  */
-static const char *cookie_add(request_rec *r, apr_pool_t *subpool, const mu_config *const conf)
+static const char *cookie_add(request_rec *r, apr_pool_t *subpool)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	const char	*new_cookie_string = NULL;
 	const char	*args = conf->custom_list;
 	char		*p = NULL;
@@ -364,11 +364,11 @@ static const char *cookie_add(request_rec *r, apr_pool_t *subpool, const mu_conf
  *
  * @param r the apache request
  * @param subpool the current subpool to use for mem allocations
- * @param conf the current configuration flags
  * @return a string to store in the cookie
  */
-static const char *cookie_addall(request_rec *r, apr_pool_t *subpool, mu_config *const conf)
+static const char *cookie_addall(request_rec *r, apr_pool_t *subpool)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	const mu_ent	*custom = NULL;
 	const char 	*new_cookie_string = NULL;
 	char		*codeduri = NULL;
@@ -382,7 +382,7 @@ static const char *cookie_addall(request_rec *r, apr_pool_t *subpool, mu_config
 
 	new_cookie_string = find_or_create_playlist(r, subpool);
 	
-	make_music_entry(r, subpool, &cookie_pack, conf, NULL, MI_RECURSIVE);
+	make_music_entry(r, subpool, &cookie_pack, NULL, MI_RECURSIVE);
 	listsort(&cookie_pack, conf->order);	/* this sets cookie_pack.fhead */
 
 	for (custom = cookie_pack.fhead; custom; custom = custom->next) {
@@ -410,11 +410,11 @@ static const char *cookie_addall(request_rec *r, apr_pool_t *subpool, mu_config
  *
  * @param r the apache request
  * @param subpool the current subpool to use for mem allocations
- * @param conf the current configuration flags (including the post arguments)
  * @return the new cookie string deprived of matching files
  */
-static const char *cookie_remove(request_rec *r, apr_pool_t *subpool, const mu_config *const conf)
+static const char *cookie_remove(request_rec *r, apr_pool_t *subpool)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	const char	*new_cookie_string = NULL, *p = NULL, *args;
 	char		*rlist;
 
@@ -452,8 +452,9 @@ static const char *cookie_remove(request_rec *r, apr_pool_t *subpool, const mu_c
 	return new_cookie_string;
 }
 
-void cookie_and_stream_work(request_rec *r, mu_config *const conf)
+void cookie_and_stream_work(request_rec *r)
 {
+	mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	const char *end_string = NULL, *args = NULL;
 	apr_pool_t *subpool = NULL;
 
@@ -467,13 +468,13 @@ void cookie_and_stream_work(request_rec *r, mu_config *const conf)
 	
 	switch (conf->options & MI_ALLOPS) {
 		case MI_COOKIEADDALL:
-			args = cookie_addall(r, subpool, conf);
+			args = cookie_addall(r, subpool);
 			break;
 		case MI_COOKIEADDLST:
-			args = cookie_add(r, subpool, conf);
+			args = cookie_add(r, subpool);
 			break;
 		case MI_COOKIEDELLST:
-			args = cookie_remove(r, subpool, conf);
+			args = cookie_remove(r, subpool);
 			break;
 		case MI_COOKIEPURGE:
 			args = apr_pstrdup(subpool, "playlist=");
