@@ -2,7 +2,7 @@
  *  html.h
  *  mod_musicindex
  *
- *  $Id: http.h 566 2006-02-04 01:14:20Z boudinr $
+ *  $Id: http.h 1003 2012-07-31 09:53:17Z varenet $
  *
  *  Created by Regis BOUDIN on Sun Jun 13 2004.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -19,9 +19,9 @@
 
 #include "mod_musicindex.h"
 
-void treat_get_args(request_rec *r, mu_config *const conf);
-void treat_post_args(request_rec *r, mu_config *const conf);
+void treat_get_args(request_rec *r);
+void treat_post_args(request_rec *r);
 
-void cookie_and_stream_work(request_rec *r, mu_config *const conf);
+void cookie_and_stream_work(request_rec *r);
 
 #endif	/* HTTP_H */
