@@ -2,11 +2,11 @@
  *  mod_musicindex.c
  *  mod_musicindex
  *
- *  $Id: mod_musicindex.c 973 2012-02-16 13:43:14Z varenet $
+ *  $Id: mod_musicindex.c 1012 2012-08-08 08:58:25Z varenet $
  *
  *  Created by Regis BOUDIN on Sat Dec 28 2002.
  *  Copyright (c) 2002-2006 Regis BOUDIN
- *  Copyright (c) 2002-2005,2007-2008,2010-2011 Thibaut VARENE
+ *  Copyright (c) 2002-2005,2007-2008,2010-2012 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -131,9 +131,9 @@
  * This file is the core of the module. It contains Apache's mandatory stuff.
  *
  * @author Regis Boudin
- * @version $Revision: 973 $
+ * @version $Revision: 1012 $
  * @date 2002-2008
- * @date 2010-2011
+ * @date 2010-2012
  *
  * http://httpd.apache.org/docs-2.0/developer/API.html
  * http://httpd.apache.org/dev/apidoc/
@@ -188,13 +188,13 @@
 /** mime types handled */
 static const char *handlers[] = {
 	"audio/mpeg",
-	"application/ogg",
-	"audio/ogg",
-	"audio/x-ogg", /* At some point, we should be able to remove this one */
+	"audio/mp4",
 	"audio/flac",
+	"audio/ogg",
 	"audio/x-flac",
 	"application/x-flac",
-	"audio/mp4",
+	"application/ogg",
+	"audio/x-ogg", /* At some point, we should be able to remove this one */
 	NULL
 };
 #else
@@ -242,21 +242,21 @@ static int handle_musicindex(request_rec *r)
 		.fsize = 0,
 	};
 
-	STRUCTTV tvbegin;
+	STRUCTTV tvbegin, tvprocess;
 	
 #ifdef HAVE_GETTIMEOFDAY
+	struct timeval tvend;
 	gettimeofday(&tvbegin, NULL);
 #endif
 
-	/* if the the module is not active, or if request is not a GET or a POST,
-	   let the dir to another module */
+	r->allowed |= ((1 << M_GET) | (1 << M_POST));
+
+	/* if the the module is not active, decline the request */
 	if (!(conf->options & MI_ACTIVE))
 		return DECLINED;
-	if ((r->method_number != M_GET) && (r->method_number != M_POST))
-		return DECLINED;
 
 #ifdef BUILD_FOR_APACHE2
-	if(strcmp(r->handler, DIR_MAGIC_TYPE))
+	if(!r->handler || strcmp(r->handler, DIR_MAGIC_TYPE))
 		return DECLINED;
 #endif
 
@@ -267,8 +267,6 @@ static int handle_musicindex(request_rec *r)
 		return HTTP_FORBIDDEN;
 	}
 
-	r->allowed |= ((1 << M_GET) | (1 << M_POST));
-
 	/* This part mostly comes from mod_autoindex. If the requested dir does
 	   not end with a '/', generate a new request with it */
 	if (r->uri[0] == '\0' || r->uri[strlen(r->uri) - 1] != '/') {
@@ -284,13 +282,13 @@ static int handle_musicindex(request_rec *r)
 		return HTTP_MOVED_PERMANENTLY;
 	}
 
-	/* Deal with optional arguments */
+	/* Deal with optional arguments or decline requests we don't handle */
 	switch (r->method_number) {
 		case M_GET:
-			treat_get_args(r, conf);
+			treat_get_args(r);
 			break;
 		case M_POST:
-			treat_post_args(r, conf);
+			treat_post_args(r);
 			break;
 		default:
 			return DECLINED;
@@ -302,39 +300,39 @@ static int handle_musicindex(request_rec *r)
 	/* XXX this is ugly */
 	if (conf->options & MI_RANDOMDIR) {
 		conf->options &= ~MI_RANDOMDIR;
-		send_randomdir(r, conf);
+		send_randomdir(r);
 		return HTTP_MOVED_TEMPORARILY;
 	}
 
 	/* Cache prologue can happen no later than here */
 	if (conf->cache && conf->cache->prologue)
-		conf->cache->prologue(r, conf);
+		conf->cache->prologue(r);
 
 	/* build the string containing the cookie content. We need it for
 	   webpages and requests to stream the cookie content */
 	if (((conf->options & (MI_STREAM|MI_TARBALL)) == 0) || ((conf->options & MI_STREAMRQ) == MI_STREAMCOOKIE) || ((conf->options & MI_DWNLDRQ) == MI_DWNLDCOOKIE))
-		cookie_and_stream_work(r, conf);
+		cookie_and_stream_work(r);
 
 	if (((conf->options & MI_STREAMRQ) == MI_STREAMLST) ||
 		((conf->options & MI_STREAMRQ) == MI_COOKIESTREAM) ||
 		((conf->options & MI_DWNLDRQ) == MI_DWNLDLST) ||
 		((conf->options & MI_DWNLDRQ) == MI_COOKIEDWNLD)) {
 		/* We build a custom playlist, don't bother with current directory */
-		build_custom_list(r, &master_pack, conf);
+		build_custom_list(r, &master_pack);
 	}
 	else {
 		/* Build current directory content (recursive if necessary) */
-		make_music_entry(r, r->pool, &master_pack, conf, NULL, MI_RECURSIVE);
+		make_music_entry(r, r->pool, &master_pack, NULL, MI_RECURSIVE);
 		listsort(&master_pack, conf->order);
 	}
 
 	/* Build custom link list, from conf->custom_list string */
 	if (((conf->options & MI_STREAM) == 0) && (conf->custom_list != NULL))
-		build_custom_list(r, &custom_pack, conf);
+		build_custom_list(r, &custom_pack);
 	
 	/* Cache epilogue can happen as early as here */
 	if (conf->cache && conf->cache->epilogue)
-		conf->cache->epilogue(r, conf);
+		conf->cache->epilogue(r);
 
 	/* Depending on the request type, set the HTTP headers. By default we
 	   return a web page. */
@@ -382,21 +380,28 @@ static int handle_musicindex(request_rec *r)
 	if (r->header_only)
 		return OK;
 
+#ifdef HAVE_GETTIMEOFDAY
+	/* Timing request processing */
+	gettimeofday(&tvend, NULL);
+	timersub(&tvend, &tvbegin, &tvprocess);
+#endif
+
+	/* At this point we're sending data back */
 	if (conf->options & MI_STREAM)
-		send_playlist(r, &master_pack, conf);
+		send_playlist(r, &master_pack);
 	else if (conf->options & MI_RSS)
-		send_rss(r, &master_pack, conf);
+		send_rss(r, &master_pack);
 #ifdef ENABLE_OUTPUT_ARCHIVE
 	else if (conf->options & MI_TARBALL)
 		send_tarball(r, &master_pack);
 #endif
 	else {
-		send_head(r, conf);
-		if (conf->search == NULL)
-			send_directories(r, &master_pack, conf);
-		send_tracks(r, &master_pack, conf);
-		send_customlist(r, &custom_pack, conf);
-		send_foot(r, conf, &tvbegin);
+		send_head(r);
+		if (!conf->search && conf->dir_per_line)
+			send_directories(r, &master_pack);
+		send_tracks(r, &master_pack);
+		send_customlist(r, &custom_pack);
+		send_foot(r, &tvbegin, &tvprocess);
 	}
 
 	return OK;
@@ -419,41 +424,34 @@ static int handle_musicfile(request_rec *r)
 	register unsigned short i;
 #endif
 
+	/* is this a request method we handle? */
 	if ((r->method_number != M_GET) || (!(conf->options & MI_ACTIVE)))
 		return DECLINED;
 
 #ifdef BUILD_FOR_APACHE2
+	/* is this a mime type we handle? */
+	if (!r->handler)
+		return DECLINED;
+	
 	for (i=0; handlers[i] && strcmp(r->handler, handlers[i]); i++);
 
 	if (handlers[i] == NULL)
 		return DECLINED;
 #endif
 
-	/* This deals with download requests, that is when there is a request
-	   with no get argument. We return DECLINED when :
-		- Download is allowed
-		- Stream is allowed and no icecast server is set
+	/* request response rationale:
+	   - Stream is allowed:
+		+ we have a "?stream" argument in the request, the client asks for
+		  a playlist containing a single file -> OK
+		+ we have no argument and no icecast server is set -> set Content-Length
+		  and DECLINED
+	   - Download is allowed:
+	   	+ we have no argument -> DECLINED
 	   Otherwise, we consider it is a forbidden request. Keep in mind that
 	   DECLINED just means the module won't handle the request, whereas
 	   HTTP_FORBIDDEN means the request is not allowed, no matter what. */
-	/* XXX We should handle fds by ourselves, and set the mime type according
-	   to our standards, maybe? */
-	if (r->args == NULL) {
-		if (conf->options & MI_ALLOWDWNLD)
-			return DECLINED;
 
-		if((conf->options & MI_ALLOWSTREAM) && (conf->iceserver == NULL)) {
-#if 0 /* XXX RFC 2424 */
-
-#endif
-			return DECLINED;
-		}
-
-		return HTTP_FORBIDDEN;
-	}
-
-
-	if ((conf->options & MI_ALLOWSTREAM) && (strcmp(r->args, "stream") == 0)) {
+	if (conf->options & MI_ALLOWSTREAM) {
 		mu_pack master_pack = {
 			.head = NULL,
 			.fhead = NULL,
@@ -461,29 +459,48 @@ static int handle_musicfile(request_rec *r)
 			.filenb = 0,
 		};
 
-		ap_set_content_type(r, "audio/x-mpegurl");
-		apr_table_setn(r->headers_out, "Content-Disposition",
-			"filename = \"playlist.m3u\"");
+		if (r->args && !strcmp(r->args, "stream")) {
+			ap_set_content_type(r, "audio/x-mpegurl");
+			apr_table_setn(r->headers_out, "Content-Disposition",
+				"filename = \"playlist.m3u\"");
 
-		ap_send_http_header(r);
+			ap_send_http_header(r);
 
-		if (r->header_only)
-			return OK;
+			if (r->header_only)
+				return OK;
+		}			
 
 		if (conf->cache && conf->cache->prologue)
-			conf->cache->prologue(r, conf);
+			conf->cache->prologue(r);
 
-		make_music_entry(r, r->pool, &master_pack, conf, NULL, MI_ALLOWSTREAM);
+		make_music_entry(r, r->pool, &master_pack, NULL, MI_ALLOWSTREAM);
 
 		if (conf->cache && conf->cache->epilogue)
-			conf->cache->epilogue(r, conf);
+			conf->cache->epilogue(r);
 
 		master_pack.fhead = master_pack.head;
-		send_playlist(r, &master_pack, conf);
+		
+		/* the request is trying to fetch a file - STREAM allowed so we go the extra mile */
+		if (!r->args && !conf->iceserver) {
+			/* RFC 3803 */
+			apr_table_setn(r->headers_out, "Content-Duration", 
+				apr_psprintf(r->pool, "%hu", master_pack.fhead->length));
 
-		return OK;
+			return DECLINED; /* let Apache deal with the actual file */
+		}
+		
+		if (r->args && !strcmp(r->args, "stream")) {
+			send_playlist(r, &master_pack);
+
+			return OK;
+		}
 	}
 
+	/* trying to fetch a file, STREAM disabled, let's see if DWNLD is allowed */
+	if ((!r->args) && (conf->options & MI_ALLOWDWNLD))
+		return DECLINED;
+
+	/* all else failing, we deny the request */
 	return HTTP_FORBIDDEN;
 }
 
