@@ -2,11 +2,11 @@
  *  mod_musicindex.h
  *  mod_musicindex
  *
- *  $Id: mod_musicindex.h 968 2011-11-01 00:20:47Z varenet $
+ *  $Id: mod_musicindex.h 1013 2012-09-01 17:42:41Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2006 Regis BOUDIN
- *  Copyright (c) 2003-2011 Thibaut VARENE
+ *  Copyright (c) 2003-2012 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -23,6 +23,11 @@
 #include <http_protocol.h>
 #include <http_log.h>
 
+/* AP24 bw compat 2.0/2.2 */
+#ifdef APLOG_USE_MODULE
+APLOG_USE_MODULE(musicindex);
+#endif
+
 /* Define to the address where bug reports for this package should be sent. */
 #undef PACKAGE_BUGREPORT
 
@@ -49,10 +54,13 @@
 
 /*
  * Compile time configuration options
+ * Comment explains what happens when they are #define'd
+ * TODO some of these should be ./configure'able
  */
-#undef NO_TITLE_STRIP	/* Disable pretty printing for untagged files */
+#undef NO_TITLE_STRIP	/* Disable title pretty printing for untagged files */
 #undef NO_BR_NOMINAL	/* Disable display of nominal bitrate when available */
-#undef DEBUG		/* Disable debug output */
+#undef BE_INSECURE	/* Enable insecure auth credential leaking in urls */
+#undef DEBUG		/* Enable debug output */
 
 /* these are not defined on BSD */
 #ifndef FALSE
@@ -258,6 +266,9 @@ enum {
   /* nada */
  #elif defined(AP22)
   #define ap_http_method ap_http_scheme
+ #elif defined(AP24)
+  #define ap_http_method ap_http_scheme
+  #define ap_get_server_name(r) ap_get_server_name_for_url(r)
  #else
   #error "unknown apache API"
  #endif
@@ -274,6 +285,10 @@ enum {
   #undef APLOG_WARNING
   #define APLOG_WARNING LOG_WARNING, 0	/**< dirty trick because ap_log_rerror takes one more arg in apache2 */
  #endif
+ #ifdef APLOG_NOTICE
+  #undef APLOG_NOTICE
+  #define APLOG_NOTICE LOG_NOTICE, 0	/**< dirty trick because ap_log_rerror takes one more arg in apache2 */
+ #endif
  #ifdef APLOG_DEBUG
   #undef APLOG_DEBUG
   #define APLOG_DEBUG LOG_DEBUG, 0	/**< dirty trick because ap_log_rerror takes one more arg in apache2 */
@@ -283,16 +298,6 @@ enum {
 
 #endif	/* ! __BUILD_FOR_APACHE2 */
 
-/**
- * first struct to handle yet unparsed files.
- * DEV NOTE: filename is the path to the file on the filesystem,
- * while uri is the path to the file on the webserver.
- */
-typedef struct mu_ent_names {
-	char filename[MAX_STRING];	/**< file path without trailing '/' */
-	char uri[MAX_STRING];		/**< file URI with trailing '/' */
-} mu_ent_names;
-
 /** 
  * Music entry. Contains various data about an associated file.
  * Special care has been taken toward typing to avoid memory waste.
@@ -344,8 +349,8 @@ typedef struct {
 	    cache. If return is !NULL, (cache_backend::readdir()) will be
 	    subsequently called on the provided return pointer.
 	    See musicindex_opendir() for details. */
-	void* (*opendir)(request_rec *r, mu_pack *const pack, const mu_config *const conf,
-			const mu_ent_names * const names, unsigned long soptions);
+	void* (*opendir)(request_rec *r, mu_pack *const pack,
+			const char * const filename, const char * const uri, unsigned long soptions);
 
 	/** If the above call provided a dir handler, this function will be used.
 	    Its primary intent is to provide a way for the cache backend to let
@@ -355,56 +360,56 @@ typedef struct {
 	    file in the current directory to be processed, terminating with NULL,
 	    a la readdir. The only nitpick here is that files listed here will
 	    still hit make_cache_entry(), that is the following cache_backend::make_entry() */
-	const char* (*readdir)(void *dir, const mu_config *const conf);
+	const char* (*readdir)(void *dir);
 
 	/** At the end of directory processing, this function is called, still
 	    mimicing opendir/readdir/closedir. */
-	void (*closedir)(void *dir, const mu_config *const conf);
+	void (*closedir)(void *dir);
 	
 	/** This call reads cached metadata for a given file */
 	mu_ent* (*make_entry)(request_rec *r, apr_pool_t *pool, FILE *const in,
-		const mu_config *const conf, mu_ent_names *const names);
+		const char *const filename);
 
 	/** This call tells the cache backend to write metadata for a given file. */
-	void (*write)(request_rec *r, const mu_ent *const entry, const mu_config *const conf,
-				  const mu_ent_names * const names);
+	void (*write)(request_rec *r, const mu_ent *const entry,
+				  const char * const name);
 
 	/** This call is guaranteed to run before any other cache backend function
 	    is called anywhere in the code path. */ 
-	void (*prologue)(request_rec *r, mu_config *const conf);
+	void (*prologue)(request_rec *r);
 
 	/** This call is guaranteed to run after any other cache backend function
 	    is called in the current module execution */
-	void (*epilogue)(request_rec *r, mu_config *const conf);
+	void (*epilogue)(request_rec *r);
 } cache_backend;
 
 /** Configuration handling structure. We use it to share configuration
  * throughout the whole module */
 struct mu_config {
-	const char * restrict title;	/**< title of the page */
-
-	const char * restrict directory;	/**< directory where musicindex files are stored */
-
-	const char * restrict css;	/**< path to the css */
-
+	/* non settings, these are piggy-backed on the config struct */
 	char * restrict search;		/**< string to search */
-
-	const char * restrict iceserver;	/**< adress of an icecast server */
-
 	const char * restrict custom_list;	/**< custom playlist */
 
+	/* actual settings */
+	const char * restrict title;	/**< title of the page */
+	const char * restrict directory;	/**< directory where musicindex files are stored */
+	const char * restrict css;	/**< path to the css */
+	const char * restrict iceserver;	/**< adress of an icecast server */
+
 	const cache_backend * restrict cache;	/**< registered cache backend (if any) */
 	void * restrict cache_setup;	/**< cache private data (if any) */
 
-	unsigned char order[SB_MAX+1];	/**< Sort order. */
-	unsigned char fields[SB_MAX+1];	/**< Fields to display */
-
 	signed short dir_per_line;	/**< number of directories to display per line. If <0 disable "pretty folders" */
 	unsigned short cookie_life;	/**< cookie lifetime in seconds */
 	short rss_items;		/**< number of entries in the RSS file */
 
 	unsigned short options;		/**< diverse options */
 	unsigned short options_not;	/**< used to disable options */
+
+	unsigned short sets;		/**< which field are dynamically set. Used to merge configs */
+
+	unsigned char order[SB_MAX+1];	/**< Sort order. */
+	unsigned char fields[SB_MAX+1];	/**< Fields to display */
 };
 
 
@@ -424,7 +429,7 @@ struct mu_config {
 /* this could arguably use APLOG_DEBUG but in most configurations this is never logged. Use it later ;-) */
 #ifdef DEBUG
  #define mi_rdebug(format, ...) \
-	ap_log_rerror(APLOG_MARK, APLOG_WARNING, r, MI_LOG_PREFIX "(%s) " format, __func__, ## __VA_ARGS__)
+	ap_log_rerror(APLOG_MARK, APLOG_NOTICE, r, MI_LOG_PREFIX "(%s) " format, __func__, ## __VA_ARGS__)
 #else
  #define mi_rdebug(format, ...)	/* nothing */
 #endif
