@@ -2,10 +2,10 @@
  *  output-tarball.c
  *  mod_musicindex
  *
- *  $Id: output-tarball.c 899 2009-12-04 13:55:29Z varenet $
+ *  $Id: output-tarball.c 1023 2013-04-09 11:13:16Z varenet $
  *
  *  Created by Thibaut VARENE on Mon Jun 26 2006.
- *  Copyright (c) 2006-2008 Thibaut VARENE
+ *  Copyright (c) 2006-2008,2012-2013 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -18,8 +18,9 @@
  * Tarballs generation
  *
  * @author Thibaut Varene
- * @version $Revision: 899 $
+ * @version $Revision: 1023 $
  * @date 2006-2008
+ * @date 2012-2013
  *
  * This file handles on the fly tarball generation and associated operations
  *
@@ -47,13 +48,13 @@
 #endif
 #include <stdio.h>	/* fopen... */
 
+#define OT_SEND_DATA	0	/**< trigger data sending in tarball_op() */
+#define OT_SEND_SIZE	1	/**< trigger size report in tarball_op() */
+
 #define RWBUFSIZE	8192	/**< Size of our r/w buffer */
 /** Skip file if ... */
 #define QSKIP_CHECKS(q)	(!(q->flags & EF_ALLOWTARBALL))
-/** Common format settings */
-#define ARCHIVE_SET_FORMAT(a)	\
-	archive_write_set_compression_none(a); \
-	archive_write_set_format_ustar(a)
+
 
 /** Holds our client data for libarchive */
 struct cdata {
@@ -74,11 +75,21 @@ wrapwrite(struct archive *a, void *client_data, const void *buff, size_t n)
 	return (ap_rwrite(buff, n, mydata->r));
 }
 
+static ssize_t
+wrapwritesize(struct archive *a, void *client_data, const void *buff, size_t n)
+{
+	ssize_t *mysize = client_data;
+	*mysize += n;
+	return n;
+}
 /**
- * Sends tarball.
+ * Triggers tarball operation.
  *
  * @param r Apache request_rec struct to handle connection details.
  * @param pack A pack of music entries.
+ * @param op Type of operation requested.
+ * @return the size of the tarball that send_tarball() would generate
+ *	for the given set of files. Only valid with #op == OT_SEND_SIZE.
  *
  * It is a known "feature" that when tar'ing a custom playlist, the generated
  * tarball will contain the full path as published on the webserver, including
@@ -86,33 +97,39 @@ wrapwrite(struct archive *a, void *client_data, const void *buff, size_t n)
  * reason not to put all the files at the root of one big tarball: in case there
  * are name doublons.
  */
-void send_tarball(request_rec *r, const mu_pack *const pack)
+static ssize_t
+tarball_op(request_rec *r, const mu_pack *const pack, int op)
 {
 	const mu_ent *q = pack->fhead;
-	struct cdata *mydata;
+	ssize_t tbsize = 0;
+	struct cdata mydata;
 	struct archive *a;
 	struct archive_entry *entry;
 	struct stat st;
 	static char buff[RWBUFSIZE];
 	int len;
 	FILE *file;
+	off_t fsize;
 
 	if (!q)
-		return;
-
-	if (!(mydata = malloc(sizeof(struct cdata))))
-		return;
+		return 0;
 
 	if (!(a = archive_write_new()))
-		goto fail1;
+		return 0;
 	
 	if (!(entry = archive_entry_new()))
-		goto fail2;
+		goto fail;
 
-	mydata->r = r;
-	ARCHIVE_SET_FORMAT(a);
-	archive_write_open(a, mydata, wrapopen, wrapwrite, NULL);
+	archive_write_set_compression_none(a);
+	archive_write_set_format_ustar(a);
 
+	if (op == OT_SEND_DATA) {
+		mydata.r = r;
+		archive_write_open(a, &mydata, wrapopen, wrapwrite, NULL);
+	}
+	else /* OT_SEND_SIZE */
+		archive_write_open(a, &tbsize, NULL, wrapwritesize, NULL);
+	
 	for (; q; q = q->next) {
 		if (unlikely(QSKIP_CHECKS(q)))
 			continue;
@@ -123,29 +140,42 @@ void send_tarball(request_rec *r, const mu_pack *const pack)
 		archive_entry_set_pathname(entry, q->file);
 		archive_write_header(a, entry);
 
-		file = fopen(q->filename, "r");
-		len = fread(buff, 1, sizeof(buff), file);
-
-		while (len > 0) {
-			archive_write_data(a, buff, len);
+		if (op == OT_SEND_DATA) {
+			file = fopen(q->filename, "r");
 			len = fread(buff, 1, sizeof(buff), file);
+			/* NB: size of buffer is 3rd arg, 1-byte long elements, otherwise everything fucks up */
+			while (len > 0) {
+				archive_write_data(a, buff, len);
+				len = fread(buff, 1, sizeof(buff), file);
+			}
+			fclose(file);
+		}
+		else {	/* OT_SEND_SIZE */
+			fsize = q->size;
+			len = (likely(fsize > sizeof(buff)) ? sizeof(buff) : fsize);
+			while (len > 0) {
+				fsize -= archive_write_data(a, buff, len);
+				len = (likely(fsize > sizeof(buff)) ? sizeof(buff) : fsize);
+			}
 		}
-		fclose(file);
 	}
 
 	archive_entry_free(entry);
-fail2:
+fail:
 	archive_write_finish(a);
-fail1:
-	free(mydata);
+
+	return tbsize;
 }
 
-static ssize_t
-wrapwritesize(struct archive *a, void *client_data, const void *buff, size_t n)
+/**
+ * Sends tarball.
+ *
+ * @param r Apache request_rec struct to handle connection details.
+ * @param pack A pack of music entries.
+ */
+void send_tarball(request_rec *r, const mu_pack *const pack)
 {
-	ssize_t *mysize = client_data;
-	*mysize += n;
-	return n;
+	tarball_op(r, pack, OT_SEND_DATA);
 }
 
 /**
@@ -153,56 +183,11 @@ wrapwritesize(struct archive *a, void *client_data, const void *buff, size_t n)
  *
  * @param r Apache request_rec struct to handle connection details.
  * @param pack A pack of music entries.
- *
- * @warning This function must be kept coherent with send_tarball()
- *
  * @return the size of the tarball that send_tarball() would generate
  *	for the given set of files.
  */
 ssize_t
 tarball_size(request_rec *r, const mu_pack *const pack)
 {
-	const mu_ent *q = pack->fhead;
-	ssize_t tbsize = -1;
-	struct archive *a;
-	struct archive_entry *ae;
-	struct stat st;
-	static char buff[RWBUFSIZE];
-	int len;
-	off_t fsize;
-
-	if (!q)
-		goto fail1;
-
-	if (!(a = archive_write_new()))
-		goto fail1;
-
-	if (!(ae = archive_entry_new()))
-		goto fail2;
-	
-	ARCHIVE_SET_FORMAT(a);
-	archive_write_open(a, &tbsize, NULL, wrapwritesize, NULL);
-
-	for (; q; q = q->next) {
-		if (QSKIP_CHECKS(q))
-			continue;
-
-		stat(q->filename, &st);
-		archive_entry_clear(ae);
-		archive_entry_copy_stat(ae, &st);
-		archive_entry_set_pathname(ae, q->file);
-		archive_write_header(a, ae);
-
-		fsize = q->size;
-		len = (likely(fsize > sizeof(buff)) ? sizeof(buff) : fsize);
-		while (len > 0) {
-			fsize -= archive_write_data(a, buff, len);
-			len = (likely(fsize > sizeof(buff)) ? sizeof(buff) : fsize);
-		}
-	}
-	archive_entry_free(ae);
-fail2:
-	archive_write_finish(a);
-fail1:
-	return tbsize;
+	return tarball_op(r, pack, OT_SEND_SIZE);
 }
