@@ -2,7 +2,7 @@
  *  playlist-flac.c
  *  mod_musicindex
  *
- *  $Id: playlist-flac.c 899 2009-12-04 13:55:29Z varenet $
+ *  $Id: playlist-flac.c 1010 2012-08-07 13:38:34Z varenet $
  *
  *  Created by Regis BOUDIN on Thu Jun 30 2004.
  *  Copyright (c) 2004 Regis BOUDIN
@@ -20,7 +20,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 899 $
+ * @version $Revision: 1010 $
  * @date 2004-2005
  * @date 2007
  *
@@ -66,21 +66,21 @@ static inline short flac_ext_check(const char *const filename)
  *
  * @param pool Pool
  * @param in ogg file to parse (closed on normal exit)
- * @param conf MusicIndex configuration paramaters struct
- * @param names Names
+ * @param filename current filename
  * @param r Apache request_rec struct to handle log writings (debugging)
  *
  * @return When possible, struct mu_ent correctly set up, file stream closed.
  */
 mu_ent *make_flac_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	const mu_config *const conf, mu_ent_names *const names)
+	const char *const filename)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	mu_ent				*p = NULL;
 	struct stat			filestat;
 	FLAC__StreamMetadata		streaminfo;
 	FLAC__Metadata_SimpleIterator	*iterator;
 
-	if (!flac_ext_check(names->filename) || !FLAC__metadata_get_streaminfo(names->filename, &streaminfo))
+	if (!flac_ext_check(filename) || !FLAC__metadata_get_streaminfo(filename, &streaminfo))
 		return NULL;
 
 	fstat(fileno(in), &filestat);
@@ -112,7 +112,7 @@ mu_ent *make_flac_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	if (!iterator)
 		goto exit;
 
-	if (FLAC__metadata_simple_iterator_init(iterator, names->filename, true, true)) {
+	if (FLAC__metadata_simple_iterator_init(iterator, filename, true, true)) {
 		FLAC__StreamMetadata *block = NULL;
 		const FLAC__StreamMetadata_VorbisComment *vc = NULL;
 		register unsigned short i;
