@@ -2,7 +2,7 @@
  *  playlist-flac.h
  *  mod_musicindex
  *
- *  $Id: playlist-flac.h 742 2007-04-23 13:15:36Z varenet $
+ *  $Id: playlist-flac.h 1010 2012-08-07 13:38:34Z varenet $
  *
  *  Created by Regis BOUDIN on Thu Jun 30 2004.
  *  Copyright (c) 2004 Regis BOUDIN
@@ -19,5 +19,5 @@
 #include "mod_musicindex.h"
 
 mu_ent *make_flac_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	const mu_config *const conf, mu_ent_names *const names);
+	const char *const filename);
 #endif
