@@ -2,7 +2,7 @@
  *  playlist-mp3.c
  *  mod_musicindex
  *
- *  $Id: playlist-mp3.c 935 2010-05-29 23:27:24Z varenet $
+ *  $Id: playlist-mp3.c 1010 2012-08-07 13:38:34Z varenet $
  *
  *  Created by Regis BOUDIN on Thu Jan 22 2004.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -20,7 +20,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 935 $
+ * @version $Revision: 1010 $
  * @date 2003-2009
  *
  * This file contains everything needed to produce music entries from
@@ -242,8 +242,7 @@ static id3_utf8_t *utf8_id3tag_findframe(struct id3_tag *tag, const char *const
  *
  * @param pool Pool
  * @param in MP3 file to parse (closed on normal exit)
- * @param conf MusicIndex configuration paramaters struct
- * @param names Names
+ * @param filename current filename
  * @param r Apache request_rec struct to handle log writings (debugging)
  *
  * @bug Some VBR files are reported as "bitrate: 0".
@@ -251,8 +250,9 @@ static id3_utf8_t *utf8_id3tag_findframe(struct id3_tag *tag, const char *const
  * @return When possible, struct mu_ent correctly set up, file stream closed.
  */
 mu_ent *make_mp3_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	const mu_config *const conf, mu_ent_names *const names)
+	const char *const filename)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	mu_ent 			*p = NULL;
 
 	struct id3_file		*id3struct = NULL;
@@ -276,7 +276,7 @@ mu_ent *make_mp3_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 #ifdef DETECT_BY_CONTENT_NOT_BORKEN
 	if (mp3_content_check(in))
 #else
-	if (mpg123_mp3_ext_check(names->filename))
+	if (mpg123_mp3_ext_check(filename))
 #endif
 		goto exit;
 	
@@ -293,7 +293,7 @@ mu_ent *make_mp3_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	p->mtime = filestat.st_mtime;
 	
 	/* Check for an ID3 tag and read it if any */
-	if ((id3struct = id3_file_open(names->filename, ID3_FILE_MODE_READONLY))) {
+	if ((id3struct = id3_file_open(filename, ID3_FILE_MODE_READONLY))) {
 		if ((tag = id3_file_tag(id3struct)) && (tag->frames)) {
 			/* tagsize = tag->paddedsize; */
 			
@@ -357,7 +357,7 @@ mu_ent *make_mp3_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 			madread_size = fread(madinput_buffer + remaining, 1, INPUT_BUFFER_SIZE - remaining, in);
 
 			if (madread_size <= 0) {
-				mi_rdebug("maderror madread_size <= 0 on %s", names->filename);
+				mi_rdebug("maderror madread_size <= 0 on %s", filename);
 				break;
 				/* TODO traitement eventuel erreur lecture */
 			}
@@ -370,7 +370,7 @@ mu_ent *make_mp3_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 						break;
 					if (!MAD_RECOVERABLE(madstream.error)) {
 						/* TODO traitement eventuel erreur recoverable */
-						mi_rdebug("maderror unrecoverable header decode on %s", names->filename);
+						mi_rdebug("maderror unrecoverable header decode on %s", filename);
 						break;
 					}
 					if (madstream.error == MAD_ERROR_LOSTSYNC) {
@@ -398,7 +398,7 @@ mu_ent *make_mp3_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 					if (mad_frame_decode(&madframe, &madstream) == -1) {
 						if (!MAD_RECOVERABLE(madstream.error)) {
 							/* TODO traitement eventuel erreur recoverable */
-							mi_rdebug("maderror unrecoverable frame decode on %s", names->filename);
+							mi_rdebug("maderror unrecoverable frame decode on %s", filename);
 							break;
 						}
 					}
