@@ -2,7 +2,7 @@
  *  playlist-mp3.h
  *  mod_musicindex
  *
- *  $Id: playlist-mp3.h 742 2007-04-23 13:15:36Z varenet $
+ *  $Id: playlist-mp3.h 1010 2012-08-07 13:38:34Z varenet $
  *
  *  Created by Regis BOUDIN on Thu Jan 22 2004.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -20,5 +20,5 @@
 #include "mod_musicindex.h"
 
 mu_ent *make_mp3_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	const mu_config *const conf, mu_ent_names *const names);
+	const char *const filename);
 #endif
