@@ -2,7 +2,7 @@
  *  playlist-mp4.c
  *  mod_musicindex
  *
- *  $Id: playlist-mp4.c 968 2011-11-01 00:20:47Z varenet $
+ *  $Id: playlist-mp4.c 1010 2012-08-07 13:38:34Z varenet $
  *
  *  Created by Thibaut VARENE on Tue Sep  2 2004.
  *  Copyright (c) 2004-2005,2007,2010-2011 Thibaut VARENE
@@ -18,7 +18,7 @@
  * MP4 files management system.
  *
  * @author Thibaut Varene
- * @version $Revision: 968 $
+ * @version $Revision: 1010 $
  * @date 2004-2005
  * @date 2007
  * @date 2010-2011
@@ -65,16 +65,16 @@ static inline short mp4_ext_check(const char *const filename)
  * @param r Apache request_rec struct to handle log writings (debugging)
  * @param pool Pool
  * @param in MP4 file to parse (closed on normal exit)
- * @param conf MusicIndex configuration paramaters struct
- * @param names Names
+ * @param filename current filename
  *
  * @return When possible, struct mu_ent correctly set up, file stream closed.
  *
  * @todo handle old AAC files that use id3 tags.
  */
 mu_ent *make_mp4_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	const mu_config *const conf, mu_ent_names *const names)
+	const char *const filename)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	mu_ent 			*p = NULL;
 
 	struct stat		filestat;
@@ -83,13 +83,13 @@ mu_ent *make_mp4_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
 	const char		*trackType;
 	
 	/* XXX We could probably rely on the trackType test only though */
-	if (!mp4_ext_check(names->filename))
+	if (!mp4_ext_check(filename))
 		goto exit;
 		
 #ifdef MP4_DETAILS_ALL /* braindead API */
-	mp4file = MP4Read(names->filename, 0);
+	mp4file = MP4Read(filename, 0);
 #else
-	mp4file = MP4Read(names->filename);
+	mp4file = MP4Read(filename);
 #endif
 	if (mp4file == MP4_INVALID_FILE_HANDLE)
 		goto exit;
