@@ -2,7 +2,7 @@
  *  playlist-ogg.c
  *  mod_musicindex
  *
- *  $Id: playlist-vorbis.c 899 2009-12-04 13:55:29Z varenet $
+ *  $Id: playlist-vorbis.c 1010 2012-08-07 13:38:34Z varenet $
  *
  *  Created by Regis BOUDIN on Thu Jan 22 2004.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -20,7 +20,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 899 $
+ * @version $Revision: 1010 $
  * @date 2003-2005
  * @date 2007
  *
@@ -62,21 +62,21 @@ static inline short ogg_ext_check(const char *const filename)
  *
  * @param pool Pool
  * @param in ogg file to parse (closed on normal exit)
- * @param conf MusicIndex configuration paramaters struct
- * @param names Names
+ * @param filename current filename
  * @param r Apache request_rec struct to handle log writings (debugging)
  *
  * @return When possible, struct mu_ent correctly set up, file stream closed.
  */
 mu_ent *make_ogg_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	const mu_config *const conf, mu_ent_names *const names)
+	const char *const filename)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	mu_ent			*p = NULL;
 	OggVorbis_File		vf;
 	const char 		*t;
 	struct stat		filestat;
 
-	if (!ogg_ext_check(names->filename) || (ov_test(in, &vf, NULL, 0) != 0))
+	if (!ogg_ext_check(filename) || (ov_test(in, &vf, NULL, 0) != 0))
 		return NULL;
 	
 	p = NEW_ENT(pool);
