@@ -2,11 +2,11 @@
  *  playlist.c
  *  mod_musicindex
  *
- *  $Id: playlist.c 899 2009-12-04 13:55:29Z varenet $
+ *  $Id: playlist.c 1022 2012-10-15 21:33:12Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2004 Regis BOUDIN
- *  Copyright (c) 2003-2009 Thibaut VARENE
+ *  Copyright (c) 2003-2009,2012 Thibaut VARENE
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License version 2.1,
@@ -20,8 +20,9 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 899 $
+ * @version $Revision: 1022 $
  * @date 2003-2009
+ * @date 2012
  *
  * This is where the magic takes place. Here are the functions in charge of
  * analyzing the content of the folders requested by the client, and of
@@ -75,13 +76,12 @@ const struct ftype const filetype[] = {
  * @param r Apache request_rec struct
  * @param pool Pool
  * @param in file to parse (closed on normal exit)
- * @param conf MusicIndex configuration paramaters struct
- * @param names Names
+ * @param filename current filename
  *
  * @return head, whatever happens.
  */
 static inline mu_ent *make_no_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	const mu_config *const conf, mu_ent_names *const names)
+	const char *const filename)
 {
 	fclose(in);
 	return NULL;
@@ -93,16 +93,17 @@ static inline mu_ent *make_no_entry(request_rec *r, apr_pool_t *pool, FILE *cons
  * @param r Apache request_rec struct
  * @param pool Apache memory pool
  * @param in file to parse (closed on normal exit)
- * @param conf MusicIndex configuration parameters
- * @param names Names struct associated with the current in file
+ * @param filename current filename
  *
  * @return the cache data on success, head otherwise
  */
 static mu_ent *make_cache_entry(request_rec *r, apr_pool_t *pool, FILE *const in,
-	const mu_config *const conf, mu_ent_names *const names)
+	const char *const filename)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
+
 	if (conf->cache && likely(conf->cache->make_entry)) {
-		mu_ent *result = conf->cache->make_entry(r, pool, in, conf, names);
+		mu_ent *result = conf->cache->make_entry(r, pool, in, filename);
 		if (result) {
 			result->flags |= EF_INCACHE;
 			return result;
@@ -132,14 +133,14 @@ static const make_entry_ptr const make_entry[] = {
 #ifdef	ENABLE_CODEC_MP3
 	make_mp3_entry,
 #endif
-#ifdef	ENABLE_CODEC_VORBIS
-	make_ogg_entry,
-#endif
 #ifdef	ENABLE_CODEC_MP4
 	make_mp4_entry,
 #endif
 #ifdef	ENABLE_CODEC_FLAC
 	make_flac_entry,
+#endif
+#ifdef	ENABLE_CODEC_VORBIS
+	make_ogg_entry,
 #endif
 	make_no_entry,	/* MUST be last before NULL */
 	NULL
@@ -164,20 +165,21 @@ typedef struct mu_dir {
  * @param r the Apache request
  * @param dir Abstract pointer to our custom directory handler
  * @param pack pack pointer
- * @param conf Configuration of the current request
  * @param names structure containing the name of the directory
  * @param soptions the current extended options
  */
 static inline void 
-musicindex_opendir(request_rec *r, mu_dir *dir, mu_pack *const pack, const mu_config *const conf,
+musicindex_opendir(request_rec *r, mu_dir *dir, mu_pack *const pack,
 			const mu_ent_names * const names, unsigned long soptions)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
+
 	/* It's all we need to initialise. Don't add a call to memset */
 	dir->dir = NULL;
 
 	/* A cache is configured, try to read directly data from it */
 	if (conf->cache && conf->cache->opendir)
-		dir->dir = conf->cache->opendir(r, pack, conf, names, soptions);
+		dir->dir = conf->cache->opendir(r, pack, names->filename, names->uri, soptions);
 	
 	/* Cache data could not be used, fall back to the old opendir way */
 	if (dir->dir == NULL) {
@@ -192,14 +194,16 @@ musicindex_opendir(request_rec *r, mu_dir *dir, mu_pack *const pack, const mu_co
 /**
  * Get the string of the next filename to open.
  *
+ * @param r Apache request_rec struct
  * @param dir Abstract pointer to our custom directory handler
- * @param conf Module configuration (to get the current cache backend)
  *
  * @return pointer to the string of the filename, @b without any trailing /,
  *		or NULL if there is nothing left to open.
  */
-static inline const char *musicindex_readdir(mu_dir *dir, const mu_config *const conf)
+static inline const char *musicindex_readdir(request_rec *r, mu_dir *dir)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
+
 	if (dir->handled == 0 ) {
 		struct dirent *dstruct = NULL;
 	
@@ -213,7 +217,7 @@ static inline const char *musicindex_readdir(mu_dir *dir, const mu_config *const
 	else {
 		/* not all cache subsys can perform whole directory read */
 		if (conf->cache && conf->cache->readdir)
-			return conf->cache->readdir(dir->dir, conf);
+			return conf->cache->readdir(dir->dir);
 	}
 	return NULL;
 }
@@ -221,24 +225,25 @@ static inline const char *musicindex_readdir(mu_dir *dir, const mu_config *const
 /**
  * Close the open directory
  *
+ * @param r Apache request_rec struct
  * @param dir Abstract pointer to our custom directory handler
- * @param conf Module configuration (to get the current cache backend)
  */
-static inline void musicindex_closedir(mu_dir *dir, const mu_config *const conf)
+static inline void musicindex_closedir(request_rec *r, mu_dir *dir)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
+
 	/* A cache is configured, try to read directly data from it */
 	if (dir->handled == 0)
 		closedir((DIR *)(dir->dir));
 	else
 		if (conf->cache->closedir)
-			conf->cache->closedir(dir->dir, conf);
+			conf->cache->closedir(dir->dir);
 }
 
 /**
  * Check lots of details that could mean we shouldn't go through the directory.
  *
  * @param r Apache current request
- * @param conf MusicIndex configuration paramaters struct
  * @param local_options This function can be called in subdirectories for which
  *	the configuration can differ from the current dir's one. This paramater
  *	allows the caller to specify the options to check against.
@@ -246,8 +251,10 @@ static inline void musicindex_closedir(mu_dir *dir, const mu_config *const conf)
  * @return TRUE if directory descend is allowed, FALSE otherwise.
  */
 static inline short
-go_through_directory(request_rec *r, const mu_config *const conf, const unsigned short local_options)
+go_through_directory(request_rec *r, const unsigned short local_options)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
+
 	/* before dealing with a directory, sanity checks */
 	/* First, is the module enabled ? */
 	if (!(local_options & MI_ACTIVE))
@@ -291,7 +298,6 @@ go_through_directory(request_rec *r, const mu_config *const conf, const unsigned
  * @param pool Pool
  * @param r Apache request_rec struct to handle log writings (debugging)
  * @param pack A pack of music entries to populate
- * @param conf MusicIndex configuration paramaters struct
  * @param names Names
  * @param soptions bitwise special options: these are made of the extended options flag set
  *	and are used to fiddle with the algorithms (regis please docu...)
@@ -299,8 +305,9 @@ go_through_directory(request_rec *r, const mu_config *const conf, const unsigned
  * @return When possible, struct mu_ent correctly set up
  */
 void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
-	const mu_config *const conf, mu_ent_names *names, unsigned long soptions)
+	mu_ent_names *names, unsigned long soptions)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	mu_ent		*p = NULL;
 	register unsigned short	i;
 	char		*uri;
@@ -386,11 +393,11 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 			   if it's present in conf->options, e.g. when searching */
 			soptions &= ((~(MI_RECURSIVE)) | (conf->options));
 
-			if (go_through_directory(r, conf, local_options) == FALSE)
+			if (go_through_directory(r, local_options) == FALSE)
 				return;
 
 			/* Try opening the directory */
-			musicindex_opendir(r, &dir_handler, pack, conf, names, soptions);
+			musicindex_opendir(r, &dir_handler, pack, names, soptions);
 			if (NULL == dir_handler.dir)
 				return;
 
@@ -408,15 +415,15 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 			/* if a cache backend can provide full listings (during opendir()) it will return NULL
 			 * here. Otherwise, it will provide filenames "a la readdir()" for each file that's
 			 * missing in cache */
-			while ((filename = musicindex_readdir(&dir_handler, conf))) {
+			while ((filename = musicindex_readdir(r, &dir_handler))) {
 				/* Before doing some strcpy, check we want the file and there is enough space... */
 				if ((filename[0] == '.') || (strlen(filename) >= fn_max) || (strlen(filename) >= uri_max))
 					continue;
 				strcpy(fn, filename);
 				strcpy(uri, filename);
-				make_music_entry(r, pool, pack, conf, names, soptions);
+				make_music_entry(r, pool, pack, names, soptions);
 			}
-			musicindex_closedir(&dir_handler, conf);
+			musicindex_closedir(r, &dir_handler);
 			return;
 		}
 		else if (!(conf->options & MI_STREAM) && !(conf->options & MI_RSS) && !(conf->options & MI_TARBALL)) {
@@ -436,7 +443,7 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 		   it (p!=head), or until we reach the end of the array. In either
 		   case, the file will be closed (this is the magic loop) */
 		for (i=0; (!p) && make_entry[i]; i++)
-			p = make_entry[i](r, pool, in, conf, names);
+			p = make_entry[i](r, pool, in, names->filename);
 
 	}
 
@@ -500,7 +507,7 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
 	/* We put that here for cache backends which need to "see" directories;
 	 * and it's faster to retrieve p->title than to process it again. */
 	if (conf->cache && (!(p->flags & EF_INCACHE)) && likely((!(conf->options & MI_QUICKPL)) && conf->cache->write))
-		conf->cache->write(r, p, conf, names);
+		conf->cache->write(r, p, names->filename);
 	
 	/* Entry is a directory, don't go further */
 	if (p->filetype < 0) {
@@ -543,12 +550,12 @@ void make_music_entry(request_rec *r, apr_pool_t *pool, mu_pack *const pack,
  *
  * @param r Apache request_rec struct
  * @param pack A pack to which the generated list will be hooked
- * @param conf MusicIndex configuration paramaters struct
  *
  * @return a list of mu_ent on success, NULL otherwise.
  */
-void build_custom_list(request_rec *r, mu_pack *const pack, const mu_config *const conf)
+void build_custom_list(request_rec *r, mu_pack *const pack)
 {
+	const mu_config *const conf = (mu_config *)ap_get_module_config(r->per_dir_config, &musicindex_module);
 	request_rec	*subreq;
 	mu_ent_names	names;
 	const char	*args;
@@ -600,7 +607,7 @@ void build_custom_list(request_rec *r, mu_pack *const pack, const mu_config *con
 		ap_destroy_sub_req(subreq);
 		
 		/* adds entries bottom-up */
-		make_music_entry(r, r->pool, pack, conf, &names,
+		make_music_entry(r, r->pool, pack, &names,
 			(MI_CUSTOM | (local_options & MI_ALLOWFETCH)));
 	}
 	free(decodeduri);
