@@ -2,7 +2,7 @@
  *  playlist.h
  *  mod_musicindex
  *
- *  $Id: playlist.h 899 2009-12-04 13:55:29Z varenet $
+ *  $Id: playlist.h 1011 2012-08-07 20:23:39Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -27,7 +27,7 @@
  * fill it, and close the "in" file if they recognise the file.
  */
 typedef mu_ent* (*make_entry_ptr)(request_rec *, apr_pool_t *, FILE *const,
-				const mu_config *const, mu_ent_names *const);
+				const char *const);
 
 /**
  * Enumerate handlers to access relevant data in struct filetype.
@@ -48,12 +48,22 @@ struct ftype {
 };
 extern const struct ftype const filetype[];
 
+/**
+ * first struct to handle yet unparsed files.
+ * DEV NOTE: filename is the path to the file on the filesystem,
+ * while uri is the path to the file on the webserver.
+ */
+typedef struct mu_ent_names {
+	char filename[MAX_STRING];	/**< file path without trailing '/' */
+	char uri[MAX_STRING];		/**< file URI with trailing '/' */
+} mu_ent_names;
+
 #define NEW_ENT(pool) ((mu_ent *)apr_pcalloc(pool, sizeof(mu_ent)))
 
 void make_music_entry(request_rec *, apr_pool_t *,
-	mu_pack *const, const mu_config *const, mu_ent_names *, unsigned long);
+	mu_pack *const, mu_ent_names *, unsigned long);
 
-void build_custom_list(request_rec *, mu_pack *const, const mu_config *const);
+void build_custom_list(request_rec *, mu_pack *const);
 
 /** this is used in several places; make it a define to avoid duplicating code */
 #define P_FLAGS_OPTIONS(p, soptions) \
