@@ -2,7 +2,7 @@
  *  inf.c
  *  mod_musicindex
  *
- *  $Id: sort.c 812 2008-12-21 01:34:29Z varenet $
+ *  $Id: sort.c 862 2009-09-01 12:58:42Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -26,7 +26,7 @@
  *
  * @author Regis Boudin
  * @author Thibaut Varene
- * @version $Revision: 812 $
+ * @version $Revision: 862 $
  * @date 2003-2004
  * @date 2007-2008
  *
@@ -103,27 +103,27 @@
 #endif
 
 /* static declarations of local routines */
-static inline short inf_by_rand(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_track(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_posn(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_date(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_length(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_bitrate(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_freq(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_size(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_mtime(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_artist(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_album(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_title(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_filetype(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_file(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_uri(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_genre(const mu_ent *const first, const mu_ent *const second);
-static inline short inf_by_dir(const mu_ent *const first, const mu_ent *const second);
+static inline short inf_by_rand(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_track(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_posn(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_date(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_length(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_bitrate(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_freq(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_size(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_mtime(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_artist(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_album(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_title(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_filetype(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_file(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_uri(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_genre(const mu_ent *const restrict first, const mu_ent *const restrict second);
+static inline short inf_by_dir(const mu_ent *const restrict first, const mu_ent *const restrict second);
 static inline short inf_global(const mu_ent *const first, const mu_ent *const second, const unsigned char *const order);
 
 /** sort function pointer type */
-typedef short (*inf_ptr)(const mu_ent *const, const mu_ent *const);
+typedef short (*inf_ptr)(const mu_ent *const restrict, const mu_ent *const restrict);
 
 /**
  * This array is used in conjonction with #mu_config->order which acts as a hash
@@ -284,32 +284,32 @@ void sort_mergesort(mu_pack *const pack, const unsigned char *const order)
 /* All the following function should return an INCREASING order,
  * unless specified otherwise. */
 
-static inline short inf_by_rand(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_rand(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	return (rand()-(RAND_MAX/2));	/* if rand returns long, the implicit cast will add entropy ;o) */
 }
 
-static inline short inf_by_track(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_track(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	return (first->track - second->track);
 }
 
-static inline short inf_by_posn(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_posn(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	return (first->posn - second->posn);
 }
 
-static inline short inf_by_date(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_date(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	return (first->date - second->date);
 }
 
-static inline short inf_by_length(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_length(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	return (first->length - second->length);
 }
 
-static inline short inf_by_bitrate(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_bitrate(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	/* bitrate is long */
 	return ((first->bitrate <= second->bitrate) ? -1 : 1);
@@ -321,12 +321,12 @@ static inline short inf_by_bitrate(const mu_ent *const first, const mu_ent *cons
  * @param second second
  * @return order
  */
-static inline short inf_by_freq(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_freq(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	return (first->freq - second->freq);
 }
 
-static inline short inf_by_size(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_size(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	/* size is long */
 	return ((first->size <= second->size) ? -1 : 1);
@@ -340,13 +340,13 @@ static inline short inf_by_size(const mu_ent *const first, const mu_ent *const s
  * @param second second
  * @return order
  */
-static inline short inf_by_mtime(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_mtime(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	/* mtime is long */
 	return ((first->mtime < second->mtime) ? 1 : -1);
 }
 
-static inline short inf_by_artist(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_artist(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	register short ret = -1;
 	if (likely(first->artist))
@@ -356,7 +356,7 @@ static inline short inf_by_artist(const mu_ent *const first, const mu_ent *const
 	return (likely(!ret) ? strcasecmp(first->artist, second->artist) : ~ret);
 }
 
-static inline short inf_by_album(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_album(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	register short ret = -1;
 	if (likely(first->album))
@@ -367,27 +367,27 @@ static inline short inf_by_album(const mu_ent *const first, const mu_ent *const
 }
 
 /* title always exists now */
-static inline short inf_by_title(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_title(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	return strcasecmp(first->title, second->title);
 }
 
-static inline short inf_by_filetype(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_filetype(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	return (first->filetype - second->filetype);
 }
 
-static inline short inf_by_file(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_file(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	return strcasecmp(first->file, second->file);
 }
 
-static inline short inf_by_uri(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_uri(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	return strcmp(first->uri, second->uri);
 }
 
-static inline short inf_by_genre(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_genre(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
 	register short ret = -1;
 	if (likely(first->genre))
@@ -398,10 +398,10 @@ static inline short inf_by_genre(const mu_ent *const first, const mu_ent *const
 }
 
 /* XXX celle la elle meriterait un ptit commentaire... */
-static inline short inf_by_dir(const mu_ent *const first, const mu_ent *const second)
+static inline short inf_by_dir(const mu_ent *const restrict first, const mu_ent *const restrict second)
 {
-	const char *const one = first->uri;
-	const char *const two = second->uri;
+	const char *const restrict one = first->uri;
+	const char *const restrict two = second->uri;
 	unsigned char cone = 'a', ctwo = 'a';
 	register unsigned short i;
 
@@ -420,7 +420,7 @@ static inline short inf_by_dir(const mu_ent *const first, const mu_ent *const se
 }
 
 /**
- * Returns the "difference" between 2 files.
+ * Returns the "difference" between 2 entries.
  *
  * If one entry is a dir, put it first. If both are, sort them by alpahbetic
  * order. Then, compare according to each different parameter designed.
@@ -434,6 +434,8 @@ static inline short inf_by_dir(const mu_ent *const first, const mu_ent *const se
  * - Right now we're not keeping track of which entry in the array actually gave
  *   us a valid result. This might leave room for more optimization.
  * - We've switched from a 4 tests filetype check to a 3 tests one.
+ * - In order to enforce the restrict contract, this function @b MUST make sure
+ *   that first != second before calling subsequent inf_*() functions.
  *
  * @warning Values within @a order param should @b NEVER trigger an out of bound
  *	access in #sort_order_functions. Checking for that would add too much
@@ -446,10 +448,13 @@ static inline short inf_by_dir(const mu_ent *const first, const mu_ent *const se
  *
  * @return The difference between entries (<0 means first is "before" second)
  */
-static inline short inf_global(const mu_ent *const first, const mu_ent *const second, const unsigned char *const order)
+static inline short inf_global(const mu_ent *const first, const mu_ent *const second, const unsigned char *const restrict order)
 {
 	short result;
 	register unsigned short i;
+	
+	if (unlikely(first == second))
+		return 0;
 
 	if (first->filetype < 0) {
 		if (second->filetype < 0)
