@@ -2,7 +2,7 @@
  *  inf.h
  *  mod_musicindex
  *
- *  $Id: sort.h 739 2007-04-22 22:25:05Z varenet $
+ *  $Id: sort.h 873 2009-09-08 13:31:34Z varenet $
  *
  *  Created by Thibaut VARENE on Thu Mar 20 2003.
  *  Copyright (c) 2003-2004 Regis BOUDIN
@@ -14,12 +14,12 @@
  *
  */
 
-#ifndef INF_H
-#define INF_H
+#ifndef SORT_H
+#define SORT_H
 
 #include "mod_musicindex.h"
 
 #define listsort sort_mergesort
 void listsort(mu_pack *const, const unsigned char *const);
 
-#endif	/* INF_H */
+#endif	/* SORT_H */
